# ALM1000 Spectrum Analyzer-1.0.py(w)  (11-28-2015)
# For Python version 2.7.8 or greater
# With external module libpysmu (for Python version 2.7);
# NUMPY module (for Python version 2.7)
# Created by D Mercer
import atexit
from collections import defaultdict
from operator import add
import libpysmu
import math
import time
import struct
import numpy.fft

from time import gmtime, strftime
import tkFont
from Tkinter import *
from tkFileDialog import askopenfilename
from tkFileDialog import asksaveasfilename
from tkSimpleDialog import askstring
from tkMessageBox import *
import webbrowser
#
TBicon = """
R0lGODlhIAAgAHAAACH5BAEAAAIALAAAAAAgACAAgQAAAP///wAAAAAAAAJJhI+py+0PYwtBWkDp
hTnv2XlfEobjUZZnmn4se72vJMtcbYN4ruz44uORgiodsfI4Im++2M5VW81OmBbVULxiRVrUsgsO
i8fUAgA7
"""

root=Tk()
root.title("ALICE Highspeed Sampling Spectrum Analyzer-1.0.py(w) (11-28-2015)")
img = PhotoImage(data=TBicon)
root.call('wm', 'iconphoto', root._w, img)

root.minsize(100, 100)
# Values that can be modified
DBdivlist = [1, 2, 3, 5, 10, 15, 20]    # dB per division
DBdivindex = IntVar(0)      # 10 dB/div as initial value
DBdivindex.set(4)
DBlevel = IntVar(0)     # Reference level
DBlevel.set(0)

GRWN = 720                  # Width of the grid 720 default
GRHN = 390                  # Height of the grid 390 default
X0L = 37                    # Left top X value of grid
Y0T = 25                    # Left top Y value of grid

SAMPLErate = 100000         # Sample rate of the ALM1000 system 100000
Two28 = 268435456           # 2 raised to 28 power
TRACEmode = IntVar(0)       # 1 normal mode, 2 max hold mode, 3 average mode
TRACEmode.set(1)
TRACEaverage = IntVar(0)           # Number of average sweeps for average mode
TRACEaverage.set(8)
TRACEreset = True           # True for first new trace, reset max hold and averageing 
MinSamples = 64
MaxSamples = 65536
mclk = 32000000 # float(eval(Fclk.get()))
DevID = "m1k"
Vdiv = IntVar(0)
Vdiv.set(10)                # Number of vertical divisions
# ZEROstuffing = 1          # ZEROstuffing for signal interpolation between frequency samples (0=none)

# Colors that can be modified

COLORframes = "#000080"   # Color = "#rrggbb" rr=red gg=green bb=blue, Hexadecimal values 00 - ff
COLORcanvas = "#000000"   # 100% black
COLORgrid = "#808080"     # 50% Gray
COLORzeroline = "#0000ff" # 100% blue
COLORtrace1 = "#00ff00"   # 100% green
COLORtrace2 = "#ff8000"   # 100% orange
COLORtrace3 = "#00ffff"   # 100% cyan
COLORtrace4 = "#ffff00"   # 100% yellow
COLORtrace5 = "#ff00ff"   # 100% magenta
COLORtraceR1 = "#008000"   # 50% green
COLORtraceR2 = "#804000"   # 50% orange
COLORtraceR3 = "#008080"   # 50% cyan
COLORtraceR4 = "#808000"   # 50% yellow
COLORtraceR5 = "#800080"   # 50% magenta
COLORtext = "#ffffff"     # 100% white
COLORsignalband = "#ff0000" # 100% red
# Button sizes that can be modified
Buttonwidth1 = 12
Buttonwidth2 = 8

# Initialisation of general variables
STARTfrequency = IntVar(0)  # Startfrequency
STARTfrequency.set(0) 
STOPfrequency = IntVar(0)   # Stopfrequency
STOPfrequency.set(20000)
ZEROstuffing = IntVar(0) # The zero stuffing value is 2 ** ZERO stuffing, calculated on initialize
ZEROstuffing.set(1)
NSteps = IntVar(0)  # number of frequency sweep steps
NSteps.set(0)
LoopNum = 1
# Other global variables required in various routines
GRW = GRWN                      # Initialize GRW
GRH = GRHN                      # Initialize GRH

CANVASwidth = GRW + 2 * X0L     # The canvas width
CANVASheight = GRH + 80         # The canvas height

ADsignal1 = []                  # Analog trace channels

FFTbandwidth = 0                # The FFT bandwidth
FFTresultA = []                 # FFT result CHA
PhaseA = []
FFTresultB = []                 # FFT result CHB
PhaseB = []
# FFTwindow = 5             # FFTwindow 0=None (rectangular B=1), 1=Cosine (B=1.24), 2=Triangular non-zero endpoints (B=1.33),
                                # 3=Hann (B=1.5), 4=Blackman (B=1.73), 5=Nuttall (B=2.02), 6=Flat top (B=3.77)
FFTwindowname = "--"            # The FFT window name
FFTmemoryA = numpy.ones(1)       # The memory for averaging
PhaseMemoryA = numpy.ones(1)
FFTmemoryB = numpy.ones(1)       # The memory for averaging
PhaseMemoryB = numpy.ones(1)

RUNstatus = 0                   # 0 stopped, 1 start, 2 running, 3 stop now, 4 stop and restart
PowerStatus = 1
STARTsample = 0                 # The start sample used for the display, be calculated on initialize
STOPsample = 0                  # The stop sample used for the display, be calculated on initialize

SMPfftpwrTwo = IntVar(0)        # The power of two of SMPfft
SMPfftpwrTwo.set(11)
SMPfft = 2 ** SMPfftpwrTwo.get()    # Initialize

STOREtrace = False              # Store and display trace

FFTwindowshape = numpy.ones(SMPfft) # The FFT window curve

T1Vline = []                # Voltage Trace line channel A
T2Vline = []                # Voltage Trace line channel B
T1Pline = []                # Phase angle Trace line channel A - B
T2Pline = []                # Phase angle Trace line channel B - A
T1VRline = []               # V reference Trace line channel A
T2VRline = []               # V reference Trace line channel B
T1PRline = []               # Phase reference Trace line channel A - B
T2PRline = []               # Phase reference Trace line channel B - A
TMline = []                 # Math Trace
TRMline = []                # reference Math Trace
# AWG variables
AWGAAmplvalue = 0.0
AWGAOffsetvalue = 0.0
AWGAFreqvalue = 0.0
AWGAPhasevalue = 0
AWGAdelayvalue = 0
AWGADutyCyclevalue = 50
AWGAWave = 'dc'
AWGBAmplvalue = 0.0
AWGBOffsetvalue = 0.0
AWGBFreqvalue = 0.0
AWGBPhasevalue = 0
AWGBdelayvalue = 0
AWGBDutyCyclevalue = 50
AWGBWave = 'dc'
# Calibration coefficients
CHAVGain = CHBVGain = 1.0
CHAVOffset = CHBVOffset = 0.0

# ====== define libsmu funcions ======
class Smu(object):
    def __init__(self):
        atexit.register(libpysmu.cleanup)
        libpysmu.setup()
        self.load_channels()

    def load_channels(self):
        self.devices = libpysmu.get_dev_info()
        self.serials = {i: v[0] for i, v in enumerate(self.devices)}
        self.devices = [x[1] for x in self.devices]

        names = (chr(x) for x in xrange(65,91))
        channels = {names.next(): (i, v) for i, d in enumerate(self.devices)
                    for k, v in d.items()}
        self.channels = {k: Channel(k, self.serials[v[0]], v[1]) for k, v in channels.items()}

        device_channels = defaultdict(list)
        for k, v in channels.items():
            device_channels[v[0]].append(Channel(k, *v))
        self.devices = {i: Device(self.serials[i], device_channels[i]) for i, v
                        in enumerate(self.devices)}

    def ctrl_transfer(self, device, bm_request_type, b_request, wValue, wIndex,
                      data, wLength, timeout):
        data = str(data)
        if bm_request_type&0x80 == 0x80:
            if data == '0':
                data = '\x00'*wLength
        else:
            wLength = 0
        ret = libpysmu.ctrl_transfer(device, bm_request_type, b_request, wValue,
                                   wIndex, data, wLength, timeout)
        if bm_request_type&0x80 == 0x80:
            return map(ord, data)
        else:
            return ret

    def __repr__(self):
        return 'Devices: ' + str(self.devices)


class Device(object):
    def __init__(self, serial, channels):
        self.serial = serial
        self.channels = channels

    def get_samples(self, n_samples):
        """query device for a list of samples from all channels

        :param n_samples: number of samples
        :type n_samples: int
        :return: list of n samples from all the device's channels
        """
        return libpysmu.get_all_inputs(self.serial, n_samples)

    @property
    def samples(self):
        """iterator for samples from the device run in continuous mode"""
        return libpysmu.iterate_inputs(self.serial)


class Channel(object):
    def __init__(self, chan, dev_serial, signals):
        self.dev = dev_serial
        self.chan = ord(chan) - 65
        self.signals = {v: i for i, v in enumerate(signals)}

    def set_mode(self, mode):
        if mode == 'v' or mode == 'V':
            libpysmu.set_mode(self.dev, self.chan, 1)
            self.mode = 1
        elif mode == 'i' or mode == 'I':
            libpysmu.set_mode(self.dev, self.chan, 2)
            self.mode = 2
        elif mode == 'd' or mode == 'D':
            libpysmu.set_mode(self.dev, self.chan, 0)
            self.mode = 0
        else:
            raise ValueError('invalid mode')

    def arbitrary(self, *args, **kwargs):
        repeat = 0
        if 'repeat' in map(str.lower, kwargs.keys()):
            repeat = 1
        wave = map(float, reduce(add, [[s]*100*n for s, n in args]))
        return libpysmu.set_output_buffer(wave, self.dev, self.chan, self.mode, repeat)
    
    def get_samples(self, n_samples):
        """query channel for samples

        :param n_samples: number of samples
        :type n_samples: int
        :return: list of n samples from the channel
        """
        return libpysmu.get_inputs(self.dev, self.chan, n_samples)

    def constant(self, val):
        """set output to a constant waveform"""
        return libpysmu.set_output_constant(self.dev, self.chan, self.mode, val)

    def square(self, midpoint, peak, period, phase, duty_cycle):
        """set output to a square waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 1, midpoint, peak, period, phase, duty_cycle)

    def sawtooth(self, midpoint, peak, period, phase):
        """set output to a sawtooth waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 2, midpoint, peak, period, phase, 42)

    def stairstep(self, midpoint, peak, period, phase):
        """set output to a stairstep waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 3, midpoint, peak, period, phase, 42)

    def sine(self, midpoint, peak, period, phase):
        """set output to a sinusoidal waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 4, midpoint, peak, period, phase, 42)

    def triangle(self, midpoint, peak, period, phase):
        """set output to a triangle waveform"""
        return libpysmu.set_output_wave(self.dev, self.chan, self.mode, 5, midpoint, peak, period, phase, 42)

    def __repr__(self):
        return 'Device: ' + str(self.dev) + '\nChannel: ' + str(self.chan) + '\nSignals: ' + str(self.signals)
#
def BSaveScreen():
    global CANVASwidth, CANVASheight
    global COLORtext
    # ask for file name
    filename = asksaveasfilename(defaultextension = ".eps", filetypes=[("Encapsulated Postscript", "*.eps")])
    # temp change text color to black
    COLORtext = "#000000"
    UpdateScreen()
    # save postscript file
    ca.postscript(file=filename, height=CANVASheight, width=CANVASwidth, colormode='color')
    # 
    COLORtext = "#ffffff"
    UpdateScreen()
# ======= Start command routines =================
def Bnot():
    print "Routine not made yet"

def donothing():
    global ShowC1_V
    
def BSaveConfig():
    global ShowC1_V, ShowC1_P, ShowC2_V, ShowC2_P, CutDC, FminEntry, MulXEntry
    global AWGAAmplEntry, AWGAOffsetEntry, AWGAFreqEntry, AWGADutyCycleEntry
    global AWGAPhaseEntry, AWGAShape, AWGATerm, AWGAMode, NSteps, FSweepMode
    global AWGBAmplEntry, AWGBOffsetEntry, AWGBFreqEntry, AWGBDutyCycleEntry
    global AWGBPhaseEntry, AWGBShape, AWGBTerm, AWGBMode, AWGSync
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global FFTwindow, DBdivindex, DBlevel, TRACEmode, TRACEaverage, Vdiv
    global SMPfftpwrTwo, SMPfft, STARTfrequency, STOPfrequency, ZEROstuffing
    
# open Config file for Write
    filename = asksaveasfilename(defaultextension = ".cfg", filetypes=[("Config files", "*.cfg")])
    ConfgFile = open(filename, "w")
    #
    ConfgFile.write('ShowC1_V.set(' + str(ShowC1_V.get()) + ')\n')
    ConfgFile.write('ShowC1_P.set(' + str(ShowC1_P.get()) + ')\n')
    ConfgFile.write('ShowC2_V.set(' + str(ShowC2_V.get()) + ')\n')
    ConfgFile.write('ShowC2_P.set(' + str(ShowC2_P.get()) + ')\n')
    #
    ConfgFile.write('FFTwindow.set(' + str(FFTwindow.get()) + ')\n')
    ConfgFile.write('SMPfftpwrTwo.set(' + str(SMPfftpwrTwo.get()) + ')\n')
    ConfgFile.write('STARTfrequency.set(' + str(STARTfrequency.get()) + ')\n')
    ConfgFile.write('STOPfrequency.set(' + str(STOPfrequency.get()) + ')\n')
    ConfgFile.write('ZEROstuffing.set(' + str(ZEROstuffing.get()) + ')\n')
    ConfgFile.write('Vdiv.set(' + str(Vdiv.get()) + ')\n')
    ConfgFile.write('NSteps.set(' + str(NSteps.get()) + ')\n')
    ConfgFile.write('FSweepMode.set(' + str(FSweepMode.get()) + ')\n')
    #
    ConfgFile.write('DBdivindex.set(' + str(DBdivindex.get()) + ')\n')
    ConfgFile.write('DBlevel.set(' + str(DBlevel.get()) + ')\n')
    ConfgFile.write('TRACEmode.set(' + str(TRACEmode.get()) + ')\n')
    ConfgFile.write('TRACEaverage.set(' + str(TRACEaverage.get()) + ')\n')
    ConfgFile.write('CutDC.set(' + str(CutDC.get()) + ')\n')
    #
    ConfgFile.write('AWGAMode.set('+ str(AWGAMode.get()) + ')\n')
    ConfgFile.write('AWGATerm.set('+ str(AWGATerm.get()) + ')\n')
    ConfgFile.write('AWGAAmplEntry.delete(0,END)\n')
    ConfgFile.write('AWGAAmplEntry.insert(4, ' + AWGAAmplEntry.get() + ')\n')
    ConfgFile.write('AWGAOffsetEntry.delete(0,END)\n')
    ConfgFile.write('AWGAOffsetEntry.insert(4, ' + AWGAOffsetEntry.get() + ')\n')
    ConfgFile.write('AWGAFreqEntry.delete(0,END)\n')
    ConfgFile.write('AWGAFreqEntry.insert(4, ' + AWGAFreqEntry.get() + ')\n')
    ConfgFile.write('AWGAPhaseEntry.delete(0,END)\n')
    ConfgFile.write('AWGAPhaseEntry.insert(4, ' + AWGAPhaseEntry.get() + ')\n')
    ConfgFile.write('AWGADutyCycleEntry.delete(0,END)\n')
    ConfgFile.write('AWGADutyCycleEntry.insert(4, ' + AWGADutyCycleEntry.get() + ')\n')
    ConfgFile.write('AWGAShape.set(' + str(AWGAShape.get()) + ')\n')
    #
    ConfgFile.write('AWGBMode.set('+ str(AWGBMode.get()) + ')\n')
    ConfgFile.write('AWGBTerm.set('+ str(AWGBTerm.get()) + ')\n')
    ConfgFile.write('AWGBAmplEntry.delete(0,END)\n')
    ConfgFile.write('AWGBAmplEntry.insert(4, ' + AWGBAmplEntry.get() + ')\n')
    ConfgFile.write('AWGBOffsetEntry.delete(0,END)\n')
    ConfgFile.write('AWGBOffsetEntry.insert(4, ' + AWGBOffsetEntry.get() + ')\n')
    ConfgFile.write('AWGBFreqEntry.delete(0,END)\n')
    ConfgFile.write('AWGBFreqEntry.insert(4, ' + AWGBFreqEntry.get() + ')\n')
    ConfgFile.write('AWGBPhaseEntry.delete(0,END)\n')
    ConfgFile.write('AWGBPhaseEntry.insert(4, ' + AWGBPhaseEntry.get() + ')\n')
    ConfgFile.write('AWGBDutyCycleEntry.delete(0,END)\n')
    ConfgFile.write('AWGBDutyCycleEntry.insert(4, ' + AWGBDutyCycleEntry.get() + ')\n')
    ConfgFile.write('AWGBShape.set(' + str(AWGBShape.get()) + ')\n')
    #
    ConfgFile.write('AWGSync.set(' + str(AWGSync.get()) + ')\n')
    #
    ConfgFile.write('CHAVGainEntry.delete(0,END)\n')
    ConfgFile.write('CHAVGainEntry.insert(4, ' + CHAVGainEntry.get() + ')\n')
    ConfgFile.write('CHBVGainEntry.delete(0,END)\n')
    ConfgFile.write('CHBVGainEntry.insert(4, ' + CHBVGainEntry.get() + ')\n')
    ConfgFile.write('CHAVOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHAVOffsetEntry.insert(4, ' + CHAVOffsetEntry.get() + ')\n')
    ConfgFile.write('CHBVOffsetEntry.delete(0,END)\n')
    ConfgFile.write('CHBVOffsetEntry.insert(4, ' + CHBVOffsetEntry.get() + ')\n')
    #
    ConfgFile.write('FminEntry.delete(0,END)\n')
    ConfgFile.write('FminEntry.insert(0, ' + FminEntry.get() + ')\n')
    ConfgFile.write('MulXEntry.delete(0,END)\n')
    ConfgFile.write('MulXEntry.insert(0, ' + MulXEntry.get() + ')\n')
    #
    ConfgFile.close()

def BLoadConfig():
    global ShowC1_V, ShowC1_P, ShowC2_V, ShowC2_P, CutDC, FminEntry, MulXEntry
    global AWGAAmplEntry, AWGAOffsetEntry, AWGAFreqEntry, AWGADutyCycleEntry
    global AWGAPhaseEntry, AWGAShape, AWGATerm, AWGAMode, NSteps, FSweepMode
    global AWGBAmplEntry, AWGBOffsetEntry, AWGBFreqEntry, AWGBDutyCycleEntry
    global AWGBPhaseEntry, AWGBShape, AWGBTerm, AWGBMode, AWGSync
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    global FFTwindow, DBdivindex, DBlevel, TRACEmode, TRACEaverage, Vdiv
    global SMPfftpwrTwo, SMPfft, STARTfrequency, STOPfrequency, ZEROstuffing
    
    # Read configuration values from file
    filename = askopenfilename(defaultextension = ".cfg", filetypes=[("Config files", "*.cfg")])
    ConfgFile = open(filename)
    for line in ConfgFile:
        try:
            exec( line.rstrip() )
        except:
            print "Skipping " + line.rstrip()
    ConfgFile.close()
    #

def BHelp():
    # open a URL, in this case, the ALICE SA user's guide
    url = "https://wiki.analog.com/university/tools/alice-sa-1.0-users-guide"
    webbrowser.open(url,new=2)
    
def BShowCurvesAll():
    global ShowC1_V, ShowC1_P, ShowC2_V, ShowC2_P
    ShowC1_V.set(1)
    ShowC1_P.set(1)
    ShowC2_V.set(1)
    ShowC2_P.set(1)

def BShowCurvesNone():
    global ShowC1_V, ShowC1_P, ShowC2_V, ShowC2_P
    ShowC1_V.set(0)
    ShowC1_P.set(0)
    ShowC2_V.set(0)
    ShowC2_P.set(0)
    
def Bcloseexit():
    global RUNstatus
    
    RUNstatus = 0
    # close ALM and exit
    root.destroy()
    exit()
    
def BNormalmode():
    global RUNstatus
    global TRACEmode

    TRACEmode.set(1)
    if RUNstatus == 0:      # Update if stopped
        UpdateScreen()
    if RUNstatus == 2:      # Restart if running
        RUNstatus = 4
    
def BPeakholdmode():
    global RUNstatus
    global TRACEmode

    TRACEmode.set(2)
    if RUNstatus == 0:      # Update if stopped
        UpdateScreen()
    if RUNstatus == 2:      # Restart if running
        RUNstatus = 4
    
def BAveragemode():
    global RUNstatus
    global TRACEaverage
    global TRACEmode

    TRACEmode.set(3)

    s = askstring("Power averaging", "Value: " + str(TRACEaverage.get()) + "x\n\nNew value:\n(1-n)")

    if (s == None):         # If Cancel pressed, then None
        return()

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        TRACEaverage.set(v)

    if TRACEaverage.get() < 1:
        TRACEaverage.set(1)

    if RUNstatus == 0:      # Update if stopped
        UpdateScreen()
    if RUNstatus == 2:      # Restart if running
        RUNstatus = 4

def BSTOREtrace():
    global ShowC1_V, ShowC1_P, ShowC2_V, ShowC2_P, STOREtrace, ShowMath
    global T1Vline, T2Vline, T1VRline, T2VRline, TRMline, TMline
    global T1Pline, T2Pline, T1PRline, T2PRline
    global PeakxA, PeakyA, PeakxB, PeakyB, PeakdbA, PeakdbB
    global PeakxM, PeakyM, PeakMdb, PeakfreqM
    global PeakfreqA, PeakfreqB, PeakfreqRA, PeakfreqRB
    global PeakxRA, PeakyRA, PeakxRB, PeakyRB, PeakdbRA, PeakdbRB
    global PeakxRM, PeakyRM, PeakRMdb, PeakfreqRM
    
    if ShowC1_V.get() == 1:
        T1VRline = T1Vline
        PeakxRA = PeakxA
        PeakyRA = PeakyA
        PeakdbRA = PeakdbA
        PeakfreqRA = PeakfreqA
        STOREtrace = True
    if ShowC2_V.get() == 1:
        T2VRline = T2Vline
        PeakxRB = PeakxB
        PeakyRB = PeakyB
        PeakdbRB = PeakdbB
        PeakfreqRB = PeakfreqB
        STOREtrace = True
    if ShowC1_P.get() == 1:
        T1PRline = T1Pline
        STOREtrace = True
    if ShowC2_P.get() == 1:
        T2PRline = T2Pline
        STOREtrace = True
    if ShowMath.get() > 0:
        TRMline = TMline
        PeakxRM = PeakxM
        PeakyRM = PeakyM
        PeakRMdb = PeakMdb
        PeakfreqRM = PeakfreqM
        STOREtrace = True

    UpdateTrace()           # Always Update

def BCSVfile():
    STOREcsvfile()          # Store the trace as CSV file

def BScreensetup():
    global GRWN
    global GRW
    global GRHN
    global GRH
    global STOREtrace
    global Vdiv
    
    if (STOREtrace == True):
        showwarning("WARNING","Clearing stored trace first")
        STOREtrace = False
        return()

    s = askstring("Screensize", "Grid size scale factor")

    try:    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = float(s)
    except:
        s = "error"

    if s != "error":
        GRW = int(GRWN*v)
        GRH = int(GRHN*v)

    s = askstring("Divisions", "Value: " + str(Vdiv.get()) + "\n\nNew value:\n(4-20)")

    if (s == None): # If Cancel pressed, then None
        return()

    try:        # Error if for example no numeric characters or OK pressed without input (s = "")
        vs = int(s)
    except:
        s = "error"

    if s != "error":
        Vdiv.set(vs)

    if Vdiv.get() < 4:
        Vdiv.set(4)

    if Vdiv.get() > 20:
        Vdiv.set(20)
    UpdateTrace()

def BStart():
    global RUNstatus, LoopNum, PowerStatus, devx, PwrBt, DevID
    global ShowC1_V, ShowC1_P, ShowC2_V, ShowC2_P, STOREtrace, ShowMath
    
    if DevID == "No Device":
        showwarning("WARNING","No Device Plugged In!")
        return()
    else:
        if PowerStatus == 0:
            PowerStatus = 1
            PwrBt.config(bg="green",text="PWR-On")
            devx.ctrl_transfer(DevID, 0x40, 0x51, 49, 0, 0, 0, 100) # turn on analog power

        if ShowC1_V.get() == 0 and ShowC2_V.get() == 0 and ShowMath.get() == 0:
            showwarning("WARNING","Select at least one trace first")
            return()    
        if RUNstatus == 0:
            RUNstatus = 1
            LoopNum = 1
    SetAD9833()
    # UpdateScreen()          # Always Update

def BStop():
    global RUNstatus
    
    if (RUNstatus == 1):
        RUNstatus = 0
    elif (RUNstatus == 2):
        RUNstatus = 3
    elif (RUNstatus == 3):
        RUNstatus = 3
    elif (RUNstatus == 4):
        RUNstatus = 3
    UpdateScreen()          # Always Update
    
def BPower():
    global RUNstatus, PowerStatus, devx, PwrBt

    if (RUNstatus == 1):
        BStop()
    if PowerStatus == 1:
        PowerStatus = 0
        PwrBt.config(bg="red",text="PWR-Off")
        devx.ctrl_transfer(DevID, 0x40, 0x50, 49, 0, 0, 0, 100) # turn off analog power
    else:
        PowerStatus = 1
        PwrBt.config(bg="green",text="PWR-On")
        devx.ctrl_transfer(DevID, 0x40, 0x51, 49, 0, 0, 0, 100) # turn on analog power
    
def Blevel1():
    global DBlevel
    global RUNstatus

    DBlevel.set(DBlevel.get() - 1)
    
    if RUNstatus == 0:      # Update if stopped
        UpdateTrace()

def Blevel2():
    global DBlevel
    global RUNstatus

    DBlevel.set(DBlevel.get() + 1)
    
    if RUNstatus == 0:      # Update if stopped
        UpdateTrace()

def Blevel3():
    global DBlevel
    global RUNstatus

    DBlevel.set(DBlevel.get() - 10)
    
    if RUNstatus == 0:      # Update if stopped
        UpdateTrace()

def Blevel4():
    global DBlevel
    global RUNstatus

    DBlevel.set(DBlevel.get() + 10)
    
    if RUNstatus == 0:      # Update if stopped
        UpdateTrace()

def BSetup():
    global RUNstatus
    global SAMPLErate
    global T1line
    global ZEROstuffing
    
    if (RUNstatus != 0):
        showwarning("WARNING","Stop sweep first")
        return()
    
    s = askstring("Zero stuffing","For better interpolation of levels between frequency samples.\nBut increases processing time!\n\nValue: " + str(ZEROstuffing.get()) + "\n\nNew value:\n(0-5, 0 is no zero stuffing)")

    if (s == None):         # If Cancel pressed, then None
        return()

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        if v < 0:
            v = 0
        if v > 5:
            v = 5
        ZEROstuffing.set(v)

    UpdateScreen()          # Always Update    

def BStartfrequency():
    global RUNstatus, STARTfrequency, STOPfrequency
    global MulXEntry

    StartValue = float(int(STARTfrequency.get())*int(MulXEntry.get()))

    if StartValue >= 1000000:
        StartValue = float(StartValue/1000000)
        Startlabel = str(StartValue)+"MHz"
    elif StartValue >= 1000:
        StartValue = StartValue/1000
        Startlabel = str(StartValue)+"KHz"
    else:
        Startlabel = str(StartValue)+"Hz"

    s = askstring("Startfrequency: ","Value: " + Startlabel + "\n\nNew value:\n")
    
    if (s == None):         # If Cancel pressed, then None
        return()

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        v = v / int(MulXEntry.get())
        STARTfrequency.set(v)

    if STARTfrequency.get() < 0:
        STARTfrequency.set(0)

    if STOPfrequency.get() <= STARTfrequency.get():
        STOPfrequency.set( STARTfrequency.get() + 1 )

    if RUNstatus == 0:      # Update if stopped
        UpdateTrace()
    if RUNstatus == 2:      # Restart if running
        RUNstatus = 4

def BStopfrequency():
    global RUNstatus, STARTfrequency, STOPfrequency
    global MulXEntry
    
    # if (RUNstatus != 0):
    #    showwarning("WARNING","Stop sweep first")
    #    return()
    StopValue = float(int(STOPfrequency.get())*int(MulXEntry.get()))
    if StopValue >= 1000000:
        StopValue = float(StopValue/1000000)
        Stoplabel = str(StopValue)+"MHz"
    elif StopValue >= 1000:
        StopValue = StopValue/1000
        Stoplabel = str(StopValue)+"KHz"
    else:
        Stoplabel = str(StopValue)+"Hz"
    s = askstring("Stopfrequency: ","Value: " + Stoplabel + "\n\nNew value:\n")

    if (s == None):         # If Cancel pressed, then None
        return()

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        v = abs(v) / int(MulXEntry.get())
        STOPfrequency.set(v)

    if STOPfrequency.get() < 10:  # Minimum stopfrequency 10 Hz
        STOPfrequency.set(10)
        
    if STARTfrequency.get() >= STOPfrequency.get():
        STARTfrequency.set(STOPfrequency.get() - 1)

    if RUNstatus == 0:      # Update if stopped
        UpdateTrace()
    if RUNstatus == 2:      # Restart if running
        RUNstatus = 4

def BSweepsteps():
    global RUNstatus
    global NSteps, LoopNum
    
    s = askstring("Number of frequency steps: ","Value: " + str(NSteps.get()) + " \n\nNew value:\n")
    
    if (s == None):         # If Cancel pressed, then None
        return()

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        NSteps.set(v)

    if NSteps.get() < 0:
        NSteps.set(0)
    if NSteps.get() > 1000:
        NSteps.set(1000)
    if RUNstatus == 0:      # Update if stopped
        UpdateTrace()
    if RUNstatus == 2:      # Restart loop count if running
        LoopNum = 1

def Bsamples1():
    global RUNstatus
    global SMPfftpwrTwo
    global SMPfft
    global TRACEreset
    
    if (SMPfftpwrTwo.get() > 6):  # Min 64
        SMPfftpwrTwo.set(SMPfftpwrTwo.get() - 1)
        TRACEreset = True   # Reset trace peak and trace average
        SMPfft = 2 ** int(SMPfftpwrTwo.get())

    if RUNstatus == 0:      # Update if stopped
        UpdateScreen()
    if RUNstatus == 2:      # Restart if running
        RUNstatus = 4

def Bsamples2():
    global RUNstatus
    global SMPfftpwrTwo
    global SMPfft
    global TRACEreset
      
    if (SMPfftpwrTwo.get() < 16): # Max 65536
        SMPfftpwrTwo.set(SMPfftpwrTwo.get() + 1)
        TRACEreset = True   # Reset trace peak and trace average
        SMPfft = 2 ** int(SMPfftpwrTwo.get())

    if RUNstatus == 0:      # Update if stopped
        UpdateScreen()
    if RUNstatus == 2:      # Restart if running
        RUNstatus = 4

def BDBdiv1():
    global DBdivindex
    global RUNstatus
    
    if (DBdivindex.get() >= 1):
        DBdivindex.set(DBdivindex.get() - 1)

    if RUNstatus == 0:      # Update if stopped
        UpdateTrace()

def BDBdiv2():
    global DBdivindex
    global DBdivlist
    global RUNstatus
    
    if (DBdivindex.get() < len(DBdivlist) - 1):
        DBdivindex.set(DBdivindex.get() + 1)

    if RUNstatus == 0:      # Update if stopped
        UpdateTrace()

# =========== Awg functions ==================
def BAWGAAmpl(temp):
    global AWGAAmplEntry, AWGAAmplvalue

    try:
        AWGAAmplvalue = float(eval(AWGAAmplEntry.get()))
    except:
        AWGAAmplEntry.delete(0,"end")
        AWGAAmplEntry.insert(0, AWGAAmplvalue)
    #UpdateAWGA()
    
def BAWGAOffset(temp):
    global AWGAOffsetEntry, AWGAOffsetvalue

    try:
        AWGAOffsetvalue = float(eval(AWGAOffsetEntry.get()))
    except:
        AWGAOffsetEntry.delete(0,"end")
        AWGAOffsetEntry.insert(0, AWGAOffsetvalue)
    #UpdateAWGA()
    
def BAWGAFreq(temp):
    global AWGAFreqEntry, AWGAFreqvalue

    try:
        AWGAFreqvalue = float(eval(AWGAFreqEntry.get()))
    except:
        AWGAFreqEntry.delete(0,"end")
        AWGAFreqEntry.insert(0, AWGAFreqvalue)

    if AWGAFreqvalue > 20000: # max freq is 20KHz
        AWGAFreqvalue = 20000
    #UpdateAWGA()

def BAWGAPhase(temp):
    global AWGAPhaseEntry, AWGAPhasevalue

    try:
        AWGAPhasevalue = float(eval(AWGAPhaseEntry.get()))
    except:
        AWGAPhaseEntry.delete(0,"end")
        AWGAPhaseEntry.insert(0, AWGAPhasevalue)

    if AWGAPhasevalue > 360: # max phase is 360 degrees
        AWGAPhasevalue = 360
    if AWGAPhasevalue < 0: # min phase is 0 degrees
        AWGAPhasevalue = 0

def BAWGADutyCycle(temp):
    global AWGADutyCycleEntry, AWGADutyCyclevalue

    try:
        AWGADutyCyclevalue = float(eval(AWGADutyCycleEntry.get()))/100
    except:
        AWGADutyCycleEntry.delete(0,"end")
        AWGADutyCycleEntry.insert(0, AWGADutyCyclevalue)

    if AWGADutyCyclevalue > 1: # max duty cycle is 100%
        AWGADutyCyclevalue = 1
    if AWGADutyCyclevalue < 0: # min duty cycle is 0%
        AWGADutyCyclevalue = 0
    #UpdateAWGA()

def BAWGAShape():
    global AWGAShape, AWGAWave

    if AWGAShape.get() == 0:
        AWGAWave = 'dc'
    if AWGAShape.get() == 1:
        AWGAWave = 'sine' 
    if AWGAShape.get() == 2:
        AWGAWave = 'triangle'
    if AWGAShape.get() == 3:
        AWGAWave = 'sawtooth'
    if AWGAShape.get() == 4:
        AWGAWave = 'square'
    if AWGAShape.get() == 5:
        AWGAWave = 'stairstep'
    if AWGAShape.get() == 6:
        AWGAWave = 'arbitrary'
    # UpdateAWGA() 

def UpdateAWGA():
    global AWGAAmplvalue, AWGAOffsetvalue
    global AWGAFreqvalue, AWGAPhasevalue
    global AWGADutyCyclevalue, FSweepMode
    global AWGAWave, AWGAMode, AWGATerm
    global CHA, SAMPLErate, DevID, devx

    temp = 0
    BAWGAAmpl(temp)
    BAWGAOffset(temp)
    if FSweepMode.get() != 1:
        BAWGAFreq(temp)
    BAWGAPhase(temp)
    BAWGADutyCycle(temp)
    BAWGAShape()
    
    if AWGAFreqvalue > 0.0:
        AWGAperiodvalue = SAMPLErate/AWGAFreqvalue
    else:
        AWGAperiodvalue = 0.0

    if AWGAPhasevalue > 0:
        AWGAdelayvalue = AWGAperiodvalue * AWGAPhasevalue / 360
    else:
        AWGAdelayvalue = 0

    if AWGATerm.get() == 0: # Open termination
        devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set GND switch to open
    elif AWGATerm.get() == 1: # 50 Ohm termination to GND
        devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x50, 33, 0, 0, 0, 100) # set GND switch to closed
    elif AWGATerm.get() == 2: # 50 Ohm termination to +2.5 Volts
        devx.ctrl_transfer(DevID, 0x40, 0x50, 32, 0, 0, 0, 100) # set 2.5 V switch to closed
        devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set GND switch to open
        
    if AWGAWave == 'dc':
        if AWGAMode.get() == 0: # Source Voltage measure current mode
            CHA.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(AWGAOffsetvalue)
            # 
        elif AWGAMode.get() == 1: # Source current measure voltage mode
            CHA.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(AWGAOffsetvalue/1000)
            # 
        elif AWGAMode.get() == 2: # High impedance mode
            CHA.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(0.0)
            #
        elif AWGAMode.get() == 3: # Split Input / Output mode
            CHA.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
            CHA.constant(AWGAOffsetvalue/1000)

    else:
        if AWGAMode.get() == 0: # Source Voltage measure current mode
            CHA.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
     
        if AWGAMode.get() == 1: # Source current measure voltage mode
            CHA.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            AWGAOffsetvalue = AWGAOffsetvalue/1000
            AWGAAmplvalue = AWGAAmplvalue/1000
        if AWGAMode.get() == 3: # Split Input / Output mode
            CHA.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
            AWGAOffsetvalue = AWGAOffsetvalue/1000
            AWGAAmplvalue = AWGAAmplvalue/1000
        if AWGAMode.get() == 2: # High impedance mode
            CHA.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 34, 0, 0, 0, 100) # close voltage sense loop
            CHA.constant(0.0)          
        else:
            if AWGAWave == 'sine':
                CHA.sine(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'triangle':
                CHA.triangle(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'sawtooth':
                CHA.sawtooth(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'square':
                CHA.square(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue, AWGADutyCyclevalue)
            elif AWGAWave == 'stairstep':
                CHA.stairstep(AWGAOffsetvalue, AWGAAmplvalue, AWGAperiodvalue, AWGAdelayvalue)
            elif AWGAWave == 'arbitrary':
                CHA.arbitrary((0.0, 1),(1.25, 1), (2.5, 1), (3.75, 1), (5, 1), (3.75, 1), (2.5, 1), (1.25, 1), (0.0, 1))
            # 

def BAWGBAmpl(temp):
    global AWGBAmplEntry, AWGBAmplvalue

    try:
        AWGBAmplvalue = float(eval(AWGBAmplEntry.get()))
    except:
        AWGBAmplEntry.delete(0,"end")
        AWGBAmplEntry.insert(0, AWGBAmplvalue)
    # UpdateAWGB()
    
def BAWGBOffset(temp):
    global AWGBOffsetEntry, AWGBOffsetvalue

    try:
        AWGBOffsetvalue = float(eval(AWGBOffsetEntry.get()))
    except:
        AWGBOffsetEntry.delete(0,"end")
        AWGBOffsetEntry.insert(0, AWGBOffsetvalue)
    # UpdateAWGB()
    
def BAWGBFreq(temp):
    global AWGBFreqEntry, AWGBFreqvalue

    try:
        AWGBFreqvalue = float(eval(AWGBFreqEntry.get()))
    except:
        AWGBFreqEntry.delete(0,"end")
        AWGBFreqEntry.insert(0, AWGBFreqvalue)

    if AWGBFreqvalue > 20000: # max freq is 20KHz
        AWGBFreqvalue = 20000
    # UpdateAWGB()
    
def BAWGBPhase(temp):
    global AWGBPhaseEntry, AWGBPhasevalue

    try:
        AWGBPhasevalue = float(eval(AWGBPhaseEntry.get()))
    except:
        AWGBPhaseEntry.delete(0,"end")
        AWGBPhaseEntry.insert(0, AWGBPhasevalue)

    if AWGBPhasevalue > 360: # max phase is 360 degrees
        AWGBPhasevalue = 360
    if AWGBPhasevalue < 0: # min phase is 0 degrees
        AWGBPhasevalue = 0
    
def BAWGBDutyCycle(temp):
    global AWGBDutyCycleEntry, AWGBDutyCyclevalue

    try:
        AWGBDutyCyclevalue = float(eval(AWGBDutyCycleEntry.get()))/100
    except:
        AWGBDutyCycleEntry.delete(0,"end")
        AWGBDutyCycleEntry.insert(0, AWGBDutyCyclevalue)

    if AWGBDutyCyclevalue > 1: # max duty cycle is 100%
        AWGBDutyCyclevalue = 1
    if AWGBDutyCyclevalue < 0: # min duty cycle is 0%
        AWGBDutyCyclevalue = 0
    # UpdateAWGB()
    
def BAWGBShape():
    global AWGBShape, AWGBWave
    
    if AWGBShape.get() == 0:
        AWGBWave = 'dc'
    if AWGBShape.get() == 1:
        AWGBWave = 'sine'
    if AWGBShape.get() == 2:
        AWGBWave = 'triangle'
    if AWGBShape.get() == 3:
        AWGBWave = 'sawtooth'
    if AWGBShape.get() == 4:
        AWGBWave = 'square'
    if AWGBShape.get() == 5:
        AWGBWave = 'stairstep'
    if AWGBShape.get() == 6:
        AWGBWave = 'arbitrary'        
    # UpdateAWGB()
    
def UpdateAWGB():
    global AWGBAmplvalue, AWGBOffsetvalue
    global AWGBFreqvalue, AWGBPhasevalue
    global AWGBDutyCyclevalue, FSweepMode
    global AWGBWave, AWGBMode, AWGBTerm
    global CHB, SAMPLErate, DevID, devx

    temp = 0
    BAWGBAmpl(temp)
    BAWGBOffset(temp)
    if FSweepMode.get() != 2:
        BAWGBFreq(temp)
    BAWGBPhase(temp)
    BAWGBDutyCycle(temp)
    BAWGBShape()
    
    if AWGBFreqvalue > 0.0:
        AWGBperiodvalue = SAMPLErate/AWGBFreqvalue
    else:
        AWGBperiodvalue = 0.0
    
    if AWGBPhasevalue > 0:
        AWGBdelayvalue = AWGBperiodvalue * AWGBPhasevalue / 360
    else:
        AWGBdelayvalue = 0
        
    if AWGBTerm.get() == 0: # Open termination
        devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set GND switch to open
    elif AWGBTerm.get() == 1: # 50 Ohm termination to GND
        devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set 2.5 V switch to open
        devx.ctrl_transfer(DevID, 0x40, 0x50, 38, 0, 0, 0, 100) # set GND switch to closed
    elif AWGBTerm.get() == 2: # 50 Ohm termination to +2.5 Volts
        devx.ctrl_transfer(DevID, 0x40, 0x50, 37, 0, 0, 0, 100) # set 2.5 V switch to closed
        devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set GND switch to open
        
    if AWGBWave == 'dc':
        if AWGBMode.get() == 0: # Source Voltage measure current mode
            CHB.set_mode('v')
            CHB.constant(AWGBOffsetvalue)
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 1: # Source current measure Voltage mode
            CHB.set_mode('i')
            CHB.constant(AWGBOffsetvalue/1000)
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 2: # Hi impedance mode
            CHB.set_mode('d')
            CHB.constant(0.0) #
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        elif AWGBMode.get() == 3: # Split Input / Output mode
            CHB.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
            CHB.constant(AWGBOffsetvalue/1000)
    else:
        if AWGBMode.get() == 0: # Source Voltage measure current mode
            CHB.set_mode('v')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
        if AWGBMode.get() == 1: # Source current measure Voltage mode
            CHB.set_mode('i')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
            AWGBOffsetvalue = AWGBOffsetvalue/1000
            AWGBAmplvalue = AWGBAmplvalue/1000
        if AWGBMode.get() == 3: # Split Input / Output mode
            CHB.set_mode('i') # channel must be in source current mode
            devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
            AWGBOffsetvalue = AWGBOffsetvalue/1000
            AWGBAmplvalue = AWGBAmplvalue/1000
        if AWGBMode.get() == 2: # Hi impedance mode
            CHB.set_mode('d')
            devx.ctrl_transfer(DevID, 0x40, 0x50, 39, 0, 0, 0, 100) # close voltage sense loop
            CHB.constant(0.0)
        else:
            if AWGBWave == 'sine':
                CHB.sine(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'triangle':
                CHB.triangle(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'sawtooth':
                CHB.sawtooth(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'square':
                CHB.square(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue, AWGBDutyCyclevalue)
            elif AWGBWave == 'stairstep':
                CHB.stairstep(AWGBOffsetvalue, AWGBAmplvalue, AWGBperiodvalue, AWGBdelayvalue)
            elif AWGBWave == 'arbitrary':
                CHB.arbitrary((0.0, 1),(1.25, 1), (2.5, 1), (3.75, 1), (5, 1), (3.75, 1), (2.5, 1), (1.25, 1), (0.0, 1))
            # 

def BAWGEnab():
    global AWGAMode, AWGBMode, AWGSync
    global CHA, CHB

    if  AWGAMode.get() == 2:
        CHA.set_mode('d')
        CHA.constant(0.0) # 
    else:
        UpdateAWGA()

    if  AWGBMode.get() == 2:
        CHB.set_mode('d')
        CHB.constant(0.0) # 
    else:
        UpdateAWGB()
              
#    if AWGSync.get() > 0:
#    else:
#        if AWG1Enab.get() > 0:     
#        else:     
#        if AWG2Enab.get() > 0:         
#        else:
            
def BAWGSync():
    global AWGSync

#    if AWGSync.get() > 0:     
#    else:

# ============================================ Main routine ====================================================

def Analog_In():   # Read the analog input from the ALM1000 and store the data into the arrays
    global ADsignal1, FFTBuffA, FFTBuffB
    global AWGSync, AWGAMode, AWGBMode
    global AWGAFreqvalue, AWGBFreqvalue
    global NSteps, LoopNum, FSweepMode
    global STARTfrequency, STOPfrequency
    global CHA, CHB, Both, devx
    global RUNstatus, SingleShot, FSweepCont
    global SAMPLErate
    global SMPfft, OverRangeFlagA, OverRangeFlagB
    global DCA, DCB
    global CHAVGainEntry, CHBVGainEntry, CHAVOffsetEntry, CHBVOffsetEntry
    
    while (True):                                           # Main loop
        # RUNstatus = 1 
        if (RUNstatus == 1) or (RUNstatus == 2):
            # Do input probe Calibration CH1VGain, CH2VGain, CH1VOffset, CH2VOffset
            try:
                InOffA = float(eval(CHAVOffsetEntry.get()))
            except:
                CHAVOffsetEntry.delete(0,END)
                CHAVOffsetEntry.insert(0, InOffA)
            try:
                InGainA = float(eval(CHAVGainEntry.get()))
            except:
                CHAVGainEntry.delete(0,END)
                CHAVGainEntry.insert(0, InGainA)
            try:
                InOffB = float(eval(CHBVOffsetEntry.get()))
            except:
                CHBVOffsetEntry.delete(0,END)
                CHBVOffsetEntry.insert(0, InOffB)
            try:
                InGainB = float(eval(CHBVGainEntry.get()))
            except:
                CHBVGainEntry.delete(0,END)
                CHBVGainEntry.insert(0, InGainB)
            #
            INITIALIZEstart()
            # Starting acquisition This is a HACK to get around non-continous AWG mode!
            # restat AWGs if indicated
            if FSweepMode.get() > 0:
                FStep = float(STOPfrequency.get() - STARTfrequency.get()) / float(NSteps.get())
                FregPoint = (LoopNum * FStep) + float(STARTfrequency.get())
                if FSweepMode.get() == 1:
                    AWGAFreqvalue = FregPoint
                if FSweepMode.get() == 2:
                    AWGBFreqvalue = FregPoint
            if AWGSync.get() > 0:
                BAWGEnab()
            #
            SHOWsamples = SMPfft + 128 # get 128 extra samples
            time.sleep(0.005)
            ADsignal1 = Both.get_samples(SHOWsamples) # get samples for both channel A and B
            # waite to finish then return to open termination
            devx.ctrl_transfer(DevID, 0x40, 0x51, 32, 0, 0, 0, 100) # set CHA 2.5 V switch to open
            devx.ctrl_transfer(DevID, 0x40, 0x51, 33, 0, 0, 0, 100) # set CHA GND switch to open
            devx.ctrl_transfer(DevID, 0x40, 0x51, 37, 0, 0, 0, 100) # set CHB 2.5 V switch to open
            devx.ctrl_transfer(DevID, 0x40, 0x51, 38, 0, 0, 0, 100) # set CHB GND switch to open
            if AWGAMode.get() == 3: # Split Input / Output mode
                devx.ctrl_transfer(DevID, 0x40, 0x51, 34, 0, 0, 0, 100) # open voltage sense loop
            if AWGBMode.get() == 3: # Split Input / Output mode
                devx.ctrl_transfer(DevID, 0x40, 0x51, 39, 0, 0, 0, 100) # open voltage sense loop
            
            FFTBuffA = [] # Clear the FFTBuff array for trace A
            FFTBuffB = [] # Clear the FFTBuff array for trace B
            OverRangeFlagA = OverRangeFlagB = 0 # Clear over range flags
            index = 127 # skip first 128 samples
            while index < SHOWsamples:
                SPA = ADsignal1[0][index]
                VAdata = float(SPA[0])
                VA = (VAdata - InOffA) * InGainA
                FFTBuffA.append(VA)
                SPB = ADsignal1[1][index]
                VBdata = float(SPB[0])
                VB = (VBdata - InOffB) * InGainB
                FFTBuffB.append(VB)
                if VAdata >= 5.0 or VAdata <= 0.0:
                    OverRangeFlagA = 1

                if VBdata >= 5.0 or VBdata <= 0.0:
                    OverRangeFlagB = 1
                index = index + 1

            DCA = numpy.average(FFTBuffA)
            DCB = numpy.average(FFTBuffB)
            if CutDC.get() == 1:
                FFTBuffA = FFTBuffA - DCA
                FFTBuffB = FFTBuffB - DCB
            UpdateAll()                         # Update Data, trace and screen
            if SingleShot.get() == 1: # Single shot sweep is on
                RUNstatus = 0
        # RUNstatus = 3: Stop
        # RUNstatus = 4: Stop and restart
        if (RUNstatus == 3) or (RUNstatus == 4):
            if RUNstatus == 3:
                RUNstatus = 0                   # Status is stopped 
            if RUNstatus == 4:          
                RUNstatus = 1                   # Status is (re)start
            UpdateScreen()                      # UpdateScreen() call
        if FSweepMode.get() > 0:
            LoopNum = LoopNum + 1
            if LoopNum > NSteps.get():
                LoopNum = 1
                if FSweepCont.get() == 0:
                    RUNstatus = 0
        # Update tasks and screens by TKinter 
        root.update_idletasks()
        root.update()                           # update screens

def UpdateAll():        # Update Data, trace and screen
    global FFTBuffA, FFTBuffB
    global SMPfft

    if len(FFTBuffA) < SMPfft and len(FFTBuffB) < SMPfft:
        return
    
    DoFFT()             # Fast Fourier transformation
    MakeTrace()         # Update the traces
    UpdateScreen()      # Update the screen 

def UpdateTrace():      # Update trace and screen
    MakeTrace()         # Update traces
    UpdateScreen()      # Update the screen

def UpdateScreen():     # Update screen with trace and text
    MakeScreen()        # Update the screen
    root.update()       # Activate updated screens    

# define ad9833 actions
def ShiftOut(DValue,GValue):
    global devx, DevID
    # do AD9833
    binstr = bin(DValue)

    binlen = len(binstr)
    datastr = binstr[2:binlen]
    datalen = len(datastr)

    if datalen < 16:
       datastr = str.rjust(datastr , 16 , '0')
       datalen = len(datastr)
    # do MiniGen
    genstr = bin(GValue)

    genlen = len(genstr)
    genstr = genstr[2:genlen]
    genlen = len(genstr)

    if genlen < 16:
       genstr = str.rjust(genstr , 16 , '0')
       genlen = len(genstr)
    i = 1
    devx.ctrl_transfer(DevID, 0x40, 0x50, 2, 0, 0, 0, 100) # fsync to 0
    # sending 0x50 = set to 0, 0x51 = set to 1
    # PIO 0 is clock
    # PIO 1 is AD9833 data
    # PIO 2 is FSYNC
    # PIO 3 is minigen data
    while i < datalen+1:
        D1code = 0x50 + int(datastr[i-1])
        D2code = 0x50 + int(genstr[i-1])
        # print D1code
        devx.ctrl_transfer(DevID, 0x40, D1code, 1, 0, 0, 0, 100) # ad9833 data bit
        devx.ctrl_transfer(DevID, 0x40, D2code, 3, 0, 0, 0, 100) # minigen data bit
        devx.ctrl_transfer(DevID, 0x40, 0x51, 0, 0, 0, 0, 100) # sclk to 1
        devx.ctrl_transfer(DevID, 0x40, 0x50, 0, 0, 0, 0, 100) # sclk to 0
        devx.ctrl_transfer(DevID, 0x40, 0x51, 0, 0, 0, 0, 100) # sclk to 1
        i = i + 1
    devx.ctrl_transfer(DevID, 0x40, 0x51, 2, 0, 0, 0, 100) # fsync to 1
    #
def BsetAD9833(Temp):
    
    SetAD9833()
    
def SetAD9833():
    global FminEntry, MulXEntry, MiniGenEntry # , Fclk, Fout
    global Two28, mclk, Omode

    DValue = 8192 + 40 # set mode for square wave output
    GValue = 8192 + Omode.get() # set MiniGen mode
    ShiftOut(DValue,GValue)
    #
    try:
        MulX = float(eval(MulXEntry.get()))
        if MulX < 1:
            MulX = 1
    except:
        MulXEntry.delete(0,END)
        MulXEntry.insert(0, MulX)
    try:
        FminE = float(eval(FminEntry.get()))*1000
    except:
        FminEntry.delete(0,END)
        FminEntry.insert(0, FminE/1000)
    if MulX == 1:
        Fmin = FminE
    else:
        Fmin = FminE - (FminE/MulX)
    #
    try:
        Fmingen = float(eval(MiniGenEntry.get()))*1000
    except:
        MiniGenEntry.delete(0,END)
        MiniGenEntry.insert(0, Fmingen/1000)
    # do AD9833
    Freg = int((Fmin*Two28)/mclk)
    Foutstr = bin(Freg)
    Foutlen = len(Foutstr)
    datastr = Foutstr[2:Foutlen]
    datalen = len(datastr)
    if datalen < 28:
       datastr = str.rjust(datastr , 28 , '0')
       datalen = len(datastr)
    Fmsb = '0b01' + datastr[0:14]
    Flsb = '0b01' + datastr[14:]
    # do MiniGen
    Fgen = int((Fmingen*Two28)/16000000)
    Fgenstr = bin(Fgen)
    Fgenlen = len(Fgenstr)
    Genstr = Fgenstr[2:Fgenlen]
    Genlen = len(Genstr)
    if Genlen < 28:
       Genstr = str.rjust(Genstr , 28 , '0')
       Genlen = len(Genstr)
    Gmsb = '0b01' + Genstr[0:14]
    Glsb = '0b01' + Genstr[14:]

    FValue = int(eval(Flsb))
    GValue = int(eval(Glsb))
    ShiftOut(FValue,GValue)
    FValue = int(eval(Fmsb))
    GValue = int(eval(Gmsb))
    ShiftOut(FValue,GValue)

def DoFFT():            # Fast Fourier transformation
    global FFTBuffA, FFTBuffB
    global ShowC1_V, ShowC1_P, ShowC2_V, ShowC2_P
    global FFTmemoryA, FFTresultA
    global FFTmemoryB, FFTresultB
    global PhaseA, PhaseB, PhaseMemoryA, PhaseMemoryB
    global FFTwindowshape, FFTbandwidth
    global SAMPLErate, STARTfrequency
    global SMPfft, LoopNum
    # global STARTfrequency, STOPfrequency
    global STARTsample, STOPsample
    global TRACEaverage, TRACEmode, FSweepMode
    global TRACEreset
    global ZEROstuffing
    
    # T1 = time.time()              # For time measurement of FFT routine
    
    REX = []
    # Convert list to numpy array REX for faster Numpy calculations
    FFTsignal = FFTBuffA[:SMPfft]    # Take the first fft samples
    REX = numpy.array(FFTsignal)    # Make a numpy arry of the list

    # Set Analog level display value
    MAXanalog = 5.0              # MAXanalog is 5 volts for ALM1000
    REX = REX / MAXanalog

    # Do the FFT window function
    REX = REX * FFTwindowshape      # The windowing shape function only over the samples

    # Zero stuffing of array for better interpolation of peak level of signals
    ZEROstuffingvalue = int(2 ** ZEROstuffing.get())
    fftsamples = ZEROstuffingvalue * SMPfft      # Add zero's to the arrays

    # Save previous trace in memory for max or average trace
    FFTmemoryA = FFTresultA
    if TRACEmode.get() == 3:
        PhaseMemoryA = PhaseA

    # FFT with numpy 
    ALL = numpy.fft.fft(REX, n=fftsamples)  # Do FFT + zerostuffing till n=fftsamples with NUMPY  ALL = Real + Imaginary part
    PhaseA = numpy.angle(ALL)               # calculate angle
    ALL = numpy.absolute(ALL)               # Make absolute SQR(REX*REX + IMX*IMX) for VOLTAGE!
    ALL = ALL * ALL                         # Convert from Voltage to Power (P = (V*V) / R; R = 1)

    le = len(ALL)
    le = le / 2                             # Only half is used, other half is mirror
    ALL = ALL[:le]                          # So take only first half of the array
    PhaseA = PhaseA[:le]
    PhaseA = numpy.degrees(PhaseA)
    Totalcorr = float(ZEROstuffingvalue)/ fftsamples # For VOLTAGE!
    Totalcorr = Totalcorr * Totalcorr               # For POWER!
    FFTresultA = Totalcorr * ALL

    REX = []
    # Convert list to numpy array REX for faster Numpy calculations
    FFTsignal = FFTBuffB[:SMPfft]    # Take the first fft samples
    REX = numpy.array(FFTsignal)    # Make a numpy arry of the list

    # Set Analog level display value
    MAXanalog = 5.0              # MAXanalog is 5 volts for ALM1000
    REX = REX / MAXanalog

    # Do the FFT window function
    REX = REX * FFTwindowshape      # The windowing shape function only over the samples

    # Zero stuffing of array for better interpolation of peak level of signals
    ZEROstuffingvalue = int(2 ** ZEROstuffing.get())
    fftsamples = ZEROstuffingvalue * SMPfft      # Add zero's to the arrays

    # Save previous trace in memory for max or average trace
    FFTmemoryB = FFTresultB
    if TRACEmode.get() == 3:
        PhaseMemoryB = PhaseB
    # FFT with numpy 
    ALL = numpy.fft.fft(REX, n=fftsamples)  # Do FFT + zerostuffing till n=fftsamples with NUMPY  ALL = Real + Imaginary part
    PhaseB = numpy.angle(ALL)               # calculate angle
    ALL = numpy.absolute(ALL)               # Make absolute SQR(REX*REX + IMX*IMX) for VOLTAGE!
    ALL = ALL * ALL                         # Convert from Voltage to Power (P = (U*U) / R; R = 1)

    le = len(ALL)
    le = le / 2                             # Only half is used, other half is mirror
    ALL = ALL[:le]                          # So take only first half of the array
    PhaseB = PhaseB[:le]
    PhaseB = numpy.degrees(PhaseB)
    Totalcorr = float(ZEROstuffingvalue)/ fftsamples # For VOLTAGE!
    Totalcorr = Totalcorr * Totalcorr               # For POWER!
    FFTresultB = Totalcorr * ALL
    TRACEsize = len(FFTresultB)
    Fsample = float(SAMPLErate / 2) / (TRACEsize - 1)
    STARTsample = float(STARTfrequency.get()) / Fsample
    if LoopNum == 1:
        PhaseMemoryB = PhaseB
    if TRACEmode.get() == 1:      # Normal mode 1, do not change
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1

    if TRACEmode.get() == 2 and TRACEreset == False:  # Peak hold mode 2, change v to peak value
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryB[ptmax+i] = PhaseB[ptmax]
                i = i + 1
#
    if TRACEmode.get() == 3 and TRACEreset == False:  # Average mode 3, add difference / TRACEaverage to v
        FFTresultB = FFTmemoryB + (FFTresultB - FFTmemoryB) / TRACEaverage.get()
        PhaseB = PhaseMemoryB +(PhaseB - PhaseMemoryB) / TRACEaverage.get()
# 
    TRACEsize = len(FFTresultA)
    Fsample = float(SAMPLErate / 2) / (TRACEsize - 1)
    STARTsample = float(STARTfrequency.get()) / Fsample
    if LoopNum == 1:
        PhaseMemoryA = PhaseA
    if TRACEmode.get() == 1:      # Normal mode 1, do not change
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1

    if TRACEmode.get() == 2 and TRACEreset == False:  # Peak hold mode 2, change v to peak value
        if FSweepMode.get() == 1:
            ptmax = numpy.argmax(FFTresultA[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1
        if FSweepMode.get() == 2:
            ptmax = numpy.argmax(FFTresultB[STARTsample::])
            if ptmax > STARTsample:
                STARTsample = ptmax
            i = 0
            while i < 6:
                PhaseMemoryA[ptmax+i] = PhaseA[ptmax]
                i = i + 1
#
        FFTresultA = numpy.maximum(FFTresultA, FFTmemoryA)
    if TRACEmode.get() == 3 and TRACEreset == False:  # Average mode 3, add difference / TRACEaverage to v
        FFTresultA = FFTmemoryA + (FFTresultA - FFTmemoryA) / TRACEaverage.get()
        PhaseA = PhaseMemoryA +(PhaseA - PhaseMemoryA) / TRACEaverage.get()
#
    if TRACEmode.get() == 2 and TRACEreset == False:
        FFTresultB = numpy.maximum(FFTresultB, FFTmemoryB)

    TRACEreset = False          # Trace reset done

    # T2 = time.time()
    # print (T2 - T1)           # For time measurement of FFT routine

def MakeTrace():        # Update the grid and trace
    global FFTmemoryA, FFTresultA
    global FFTmemoryB, FFTresultB
    global PhaseA, PhaseB, PhaseMemoryA, PhaseMemoryB
    global ShowC1_V, ShowC1_P, ShowC2_V, ShowC2_P, ShowMath
    global PeakxA, PeakyA, PeakxB, PeakyB, PeakdbA, PeakdbB
    global PeakxM, PeakyM, PeakMdb, PeakfreqM
    global PeakfreqA, PeakfreqB
    global DBdivindex   # Index value
    global DBdivlist    # dB per division list
    global DBlevel      # Reference level
    global GRH          # Screenheight
    global GRW          # Screenwidth
    global SAMPLErate
    global STARTfrequency, STOPfrequency
    global STARTsample, STOPsample, LoopNum, FSweepMode
    global STOREtrace, TRACEmode
    global T1Vline, T2Vline, TMline, T1Pline, T2Pline
    global Vdiv         # Number of vertical divisions
    global X0L          # Left top X value
    global Y0T          # Left top Y value

    # Set the TRACEsize variable
    TRACEsize = 0
    if ShowC1_V.get() == 1 or ShowMath.get() > 0:
        TRACEsize = len(FFTresultA)     # Set the trace length
    elif ShowC2_V.get() == 1 or ShowMath.get() > 0:
        TRACEsize = len(FFTresultB)
    if TRACEsize == 0:                  # If no trace, skip rest of this routine
        return()
    if FSweepMode.get() > 0 and LoopNum == NSteps.get():
        PhaseA = PhaseMemoryA
        PhaseB = PhaseMemoryB
    # Vertical conversion factors (level dBs) and border limits
    Yconv = float(GRH) / (Vdiv.get() * DBdivlist[DBdivindex.get()])     # Conversion factors, Yconv is the number of screenpoints per dB
    Yc = float(Y0T) + Yconv * (DBlevel.get())                     # Yc is the 0 dBm position, can be outside the screen!
    Ymin = Y0T                  # Minimum position of screen grid (top)
    Ymax = Y0T + GRH            # Maximum position of screen grid (bottom)
    Yphconv = float(GRH) / 360
    Yp = float(Y0T) + Yphconv + 180
    # Horizontal conversion factors (frequency Hz) and border limits
    Fpixel = float(STOPfrequency.get() - STARTfrequency.get()) / GRW    # Frequency step per screen pixel
    Fsample = float(SAMPLErate / 2) / (TRACEsize - 1)       # Frequency step per sample   
    STARTsample = float(STARTfrequency.get()) / Fsample     # First sample in FFTresult[] that is used
    STARTsample = int(math.ceil(STARTsample))               # First within screen range

    STOPsample = float(STOPfrequency.get()) / Fsample       # Last sample in FFTresult[] that is used
    STOPsample = int(math.floor(STOPsample))                # Last within screen range, math.floor actually not necessary, part of int

    MAXsample = TRACEsize                                   # Just an out of range check
    if STARTsample > (MAXsample - 1):
        STARTsample = MAXsample - 1

    if STOPsample > MAXsample:
        STOPsample = MAXsample

    T1Vline = []
    T2Vline = []
    T1Pline = []
    T2Pline = []
    TMline = []
    n = STARTsample
    PeakfreqA = PeakfreqB = PeakfreqM = F = n * Fsample
    if ShowC1_V.get() == 1:
        PeakdbA = (10 * math.log10(float(FFTresultA[n])) + 17)
        PeakyA = Yc - Yconv * PeakdbA
    if ShowC2_V.get() == 1:
        PeakdbB = (10 * math.log10(float(FFTresultB[n])) + 17)
        PeakyB = Yc - Yconv * PeakdbB
    if ShowMath.get() > 0:
        if ShowMath.get() == 1:
            PeakMdb = PeakdbA - PeakdbB
        if ShowMath.get() == 2:
            PeakMdb = PeakdbB - PeakdbA
        PeakyM = Yc - Yconv * PeakMdb
    PeakxA = PeakxB = PeakxM = x = X0L + (F - STARTfrequency.get())  / Fpixel
    while n <= STOPsample:
        F = n * Fsample
        x = X0L + (F - STARTfrequency.get())  / Fpixel
        if ShowC1_V.get() == 1: 
            T1Vline.append(int(x + 0.5))
            try:
                dbA = (10 * math.log10(float(FFTresultA[n])) + 17)   # Convert power to DBs, except for log(0) error
                ya = Yc - Yconv * dbA  # Add 17 dB for max value of +10 dB ALSO in CSV file routine!
            except:
                ya = Ymax
            if (ya < Ymin):
                ya = Ymin
            if (ya > Ymax):
                ya = Ymax
            if dbA > PeakdbA:
                PeakdbA = dbA
                PeakyA = int(ya + 0.5)
                PeakxA = int(x + 0.5)
                PeakfreqA = F
            T1Vline.append(int(ya + 0.5))
        if ShowC2_V.get() == 1:
            T2Vline.append(int(x + 0.5))
            try:
                dbB = (10 * math.log10(float(FFTresultB[n])) + 17) # Add 17 dB for max value of +10 dB ALSO in CSV file routine!
                yb = Yc - Yconv * dbB 
            except:
                yb = Ymax
            if (yb < Ymin):
                yb = Ymin
            if (yb > Ymax):
                yb = Ymax
            if dbB > PeakdbB:
                PeakdbB = dbB
                PeakyB = int(yb + 0.5)
                PeakxB = int(x + 0.5)
                PeakfreqB = F
            T2Vline.append(int(yb + 0.5))
        if ShowC1_P.get() == 1:
            T1Pline.append(int(x + 0.5))
            if FSweepMode.get() > 0:
                RelPhase = PhaseMemoryA[n]-PhaseMemoryB[n]
            else:
                RelPhase = PhaseA[n]-PhaseB[n]
            if RelPhase > 180:
                RelPhase = RelPhase - 360
            elif RelPhase < -180:
                RelPhase = RelPhase + 360
            PhErr = 0.0018 * n * Fsample # calculate pahse error due half sample period offset
            RelPhase = RelPhase + PhErr - 12
            ya = Yp - Yphconv * RelPhase
            T1Pline.append(int(ya + 0.5))
        if ShowC2_P.get() == 1:
            T2Pline.append(int(x + 0.5))
            if FSweepMode.get() > 0:
                RelPhase = PhaseMemoryB[n]-PhaseMemoryA[n]
            else:
                RelPhase = PhaseB[n]-PhaseA[n]
            if RelPhase > 180:
                RelPhase = RelPhase - 360
            elif RelPhase < -180:
                RelPhase = RelPhase + 360
            PhErr = 0.0018 * n * Fsample # calculate pahse error due half sample period offset
            RelPhase = RelPhase - PhErr - 12
            ya = Yp - Yphconv * RelPhase
            T2Pline.append(int(ya + 0.5))
        if ShowMath.get() > 0:
            TMline.append(int(x + 0.5))
            dbA = (10 * math.log10(float(FFTresultA[n])) + 17) # Convert power to DBs, except for log(0) error
            dbB = (10 * math.log10(float(FFTresultB[n])) + 17) # Add 17 dB for max value of +10 dB ALSO in CSV file routine!
            if ShowMath.get() == 1:
                MdB = dbA - dbB
            elif ShowMath.get() == 2:
                MdB = dbB - dbA
            yb = Yc - Yconv * MdB
            if (yb < Ymin):
                yb = Ymin
            if (yb > Ymax):
                yb = Ymax
            if MdB > PeakMdb:
                PeakMdb = MdB
                PeakyM = int(yb + 0.5)
                PeakxM = int(x + 0.5)
                PeakfreqM = F
            TMline.append(int(yb + 0.5))
        n = n + 1               

def STOREcsvfile():     # Store the trace as CSV file [frequency, magnitude or dB value]
    global FFTmemoryA, FFTresultA
    global FFTmemoryB, FFTresultB
    global PhaseA, PhaseB
    global SAMPLErate, ShowC1_V, ShowC2_V
    
    # Set the TRACEsize variable
    if ShowC1_V.get() == 1:
        TRACEsize = len(FFTresultA)     # Set the trace length
    elif ShowC2_V.get() == 1:
        TRACEsize = len(FFTresultB)
    if TRACEsize == 0:                  # If no trace, skip rest of this routine
        return()
# ask if save as magnitude or dB
    s = askstring("Mag or dB: ","Save amplidude data as Mag (0) or dB (1):\n")

    if (s == None):         # If Cancel pressed, then None
        return()
    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        dB = int(s)
    except:
        dB = 1 # default to dB

    # Make the file name and open it
    tme =  strftime("%Y%b%d-%H%M%S", gmtime())      # The time
    filename = "Spectrum-" + tme
    filename = filename + ".csv"
    # open file to save data
    filename = asksaveasfilename(initialfile = filename, defaultextension = ".csv", filetypes=[("Comma Separated Values", "*.csv")])
    DataFile = open(filename,'a')  # Open output file
    HeaderString = 'Frequency-#, '
    if ShowC1_V.get() == 1:
        if dB == 1:
            HeaderString = HeaderString + 'CA-dB, '
        if dB == 0:
            HeaderString = HeaderString + 'CA-Mag, '
    if ShowC2_V.get() == 1:
        if dB == 1:
            HeaderString = HeaderString + 'CB-dB, '
        if dB == 0:
            HeaderString = HeaderString + 'CB-Mag, '
    if ShowC1_P.get() == 1:
        HeaderString = HeaderString + 'Phase A-B, '
    if ShowC2_P.get() == 1:
        HeaderString = HeaderString + 'Phase B-A, '
    HeaderString = HeaderString + '\n'
    DataFile.write( HeaderString )

    Fsample = float(SAMPLErate / 2) / (TRACEsize - 1)   # Frequency step per sample   

    n = 0
    while n < TRACEsize:
        F = n * Fsample
        txt = str(F)
        if ShowC1_V.get() == 1:
            if dB == 1:
                V = 10 * math.log10(float(FFTresultA[n])) + 17  # Add 17 dB for max value of +10 dB
            else:
                V = 50.12 * float(FFTresultA[n])# scale to Vrms
            txt = txt + "," + str(V) 
        if ShowC2_V.get() == 1:
            if dB == 1:
                V = 10 * math.log10(float(FFTresultB[n])) + 17  # Add 17 dB for max value of +10 dB
            else:
                V = 50.12 * float(FFTresultB[n]) # scale to Vrms
            txt = txt  + "," + str(V)
        if ShowC1_P.get() == 1:
            RelPhase = PhaseA[n]-PhaseB[n]
            if RelPhase > 180:
                RelPhase = RelPhase - 360
            elif RelPhase < -180:
                RelPhase = RelPhase + 360
            txt = txt + "," + str(RelPhase)
        if ShowC2_P.get() == 1:
            RelPhase = PhaseB[n]-PhaseA[n]
            if RelPhase > 180:
                RelPhase = RelPhase - 360
            elif RelPhase < -180:
                RelPhase = RelPhase + 360
            txt = txt + "," + str(RelPhase)
        txt = txt + "\n"
        DataFile.write(txt)
        n = n + 1    

    DataFile.close()                           # Close the file

def MakeScreen():       # Update the screen with traces and text
    global CANVASheight, CANVASwidth, SmoothCurves
    global PeakxA, PeakyA, PeakxB, PeakyB, PeakdbA, PeakdbB
    global PeakxM, PeakyM, PeakMdb, PeakfreqM
    global PeakfreqA, PeakfreqB, PeakfreqRA, PeakfreqRB
    global PeakxRA, PeakyRA, PeakxRB, PeakyRB, PeakdbRA, PeakdbRB
    global PeakxRM, PeakyRM, PeakRMdb, PeakfreqRM
    global COLORgrid    # The colors
    global COLORsignalband, COLORtext
    global COLORtrace1, COLORtrace2
    global FSweepMode, LoopNum
    global DBdivindex   # Index value
    global DBdivlist    # dB per division list
    global DBlevel      # Reference level
    global FFTwindow, FFTbandwidth, ZEROstuffing, FFTwindowname
    global X0L          # Left top X value
    global Y0T          # Left top Y value
    global GRW          # Screenwidth
    global GRH          # Screenheight
    global RUNstatus    # 0 stopped, 1 start, 2 running, 3 stop now, 4 stop and restart
    global SAMPLErate, SingleShot
    global SMPfft       # number of FFT samples
    global STARTfrequency, STOPfrequency, FminEntry, MulXEntry
    global STOREtrace, ShowC1_P, ShowC2_P, ShowRA_V, ShowRB_V, ShowMarker, ShowMath
    global ShowRA_P, ShowRB_P
    global T1Vline, T2Vline, T1Pline, T1VRline, T2VRline, TMline, TRMline
    global T1PRline, T2PRline
    global TRACEaverage # Number of traces for averageing
    global TRACEmode    # 1 normal 2 max 3 average
    global Vdiv         # Number of vertical divisions

    # Delete all items on the screen
    de = ca.find_enclosed ( -1000, -1000, CANVASwidth+1000, CANVASheight+1000 )    
    for n in de: 
        ca.delete(n)
 
    # Draw horizontal grid lines
    i = 0
    x1 = X0L
    x2 = X0L + GRW
    while (i <= Vdiv.get()):
        y = Y0T + i * GRH/Vdiv.get()
        Dline = [x1,y,x2,y]
        if i == 0 or i == Vdiv.get():
            ca.create_line(Dline, fill=COLORgrid)
        else:
            ca.create_line(Dline, dash=(4,3), fill=COLORgrid)
        Vaxis_value = (DBlevel.get() - (i * DBdivlist[DBdivindex.get()]))
        Vaxis_label = str(Vaxis_value)
        ca.create_text(x1-3, y, text=Vaxis_label, fill=COLORtrace1, anchor="e", font=("arial", 8 ))
        if ShowC1_P.get() == 1 or ShowC2_P.get() == 1:
            Vaxis_value = ( 180 - ( i * (360 / Vdiv.get())))
            Vaxis_label = str(Vaxis_value)
            ca.create_text(x2+3, y, text=Vaxis_label, fill=COLORtrace3, anchor="w", font=("arial", 8 ))
        i = i + 1

    # Draw vertical grid lines
    i = 0
    y1 = Y0T
    y2 = Y0T + GRH
    Freqdiv = float((int(STOPfrequency.get()) - int(STARTfrequency.get())) / 10)
    while (i < 11):
        x = X0L + i * GRW/10
        Dline = [x,y1,x,y2]
        if i == 0 or i == 10:
            ca.create_line(Dline, fill=COLORgrid)
        else:
            ca.create_line(Dline, dash=(4,3), fill=COLORgrid)
        axis_value = float(int(STARTfrequency.get()) + (i * Freqdiv))*int(MulXEntry.get())
        if axis_value >= 1000000:
            axis_value = float(axis_value/1000000)
            axis_label = str(axis_value)+"MHz"
        elif axis_value >= 1000:
            axis_value = axis_value/1000
            axis_label = str(axis_value)+"KHz"
        else:
            axis_label = str(axis_value)+"Hz"
        ca.create_text(x, y2+3, text=axis_label, fill=COLORgrid, anchor="n", font=("arial", 8 ))
        i = i + 1
    SmoothBool = SmoothCurves.get()
    # Draw traces
    if len(T1Vline) > 4:    # Avoid writing lines with 1 coordinate
        # Write the trace CHA
        if OverRangeFlagA == 1:
            ca.create_line(T1Vline, fill=COLORsignalband, smooth=SmoothBool, splinestep=5)
        else:
            ca.create_line(T1Vline, fill=COLORtrace1, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() == 1:
            PeakValue = float(PeakfreqA*int(MulXEntry.get()))
            if PeakValue >= 1000000:
                PeakValue = float(PeakValue/1000000)
                Peak_label = ' {0:.2f} '.format(PeakdbA) + ',' + ' {0:.1f} '.format(PeakValue)+ 'MHz'
            elif PeakValue >= 1000:
                PeakValue = PeakValue/1000
                Peak_label = ' {0:.2f} '.format(PeakdbA) + ',' + ' {0:.1f} '.format(PeakValue)+ 'KHz'
            else:
                Peak_label = ' {0:.2f} '.format(PeakdbA) + ',' + ' {0:.1f} '.format(PeakValue)+ 'Hz'
            ca.create_text(PeakxA, PeakyA, text=Peak_label, fill=COLORtrace1, anchor="e", font=("arial", 8 ))
    if len(T2Vline) > 4:    # Avoid writing lines with 1 coordinate
        # Write the trace CHB
        if OverRangeFlagB == 1:
            ca.create_line(T2Vline, fill=COLORsignalband, smooth=SmoothBool, splinestep=5)
        else:
            ca.create_line(T2Vline, fill=COLORtrace2, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() == 1:
            PeakValue = float(PeakfreqB*int(MulXEntry.get()))
            if PeakValue >= 1000000:
                PeakValue = float(PeakValue/1000000)
                Peak_label = ' {0:.2f} '.format(PeakdbB) + ',' + ' {0:.1f} '.format(PeakValue)+ 'MHz'
            elif PeakValue >= 1000:
                PeakValue = PeakValue/1000
                Peak_label = ' {0:.2f} '.format(PeakdbB) + ',' + ' {0:.1f} '.format(PeakValue)+ 'KHz'
            else:
                Peak_label = ' {0:.2f} '.format(PeakdbB) + ',' + ' {0:.1f} '.format(PeakValue)+ 'Hz'
            ca.create_text(PeakxB, PeakyB, text=Peak_label, fill=COLORtrace2, anchor="w", font=("arial", 8 ))
    if len(T1Pline) > 4:    # Avoid writing lines with 1 coordinate
        # Write the phase trace A-B 
        ca.create_line(T1Pline, fill=COLORtrace3, smooth=SmoothBool, splinestep=5)
    if len(T2Pline) > 4:    # Avoid writing lines with 1 coordinate
        # Write the phase trace A-B 
        ca.create_line(T2Pline, fill=COLORtrace4, smooth=SmoothBool, splinestep=5)
    if ShowRA_V.get() == 1 and len(T1VRline) > 4:   # Write the ref trace A if active
        ca.create_line(T1VRline, fill=COLORtraceR1, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() == 1:
            Peak_label = ' {0:.2f} '.format(PeakdbRA) + ',' + ' {0:.1f} '.format(PeakfreqRA)
            ca.create_text(PeakxRA, PeakyRA, text=Peak_label, fill=COLORtraceR1, anchor="e", font=("arial", 8 ))
    if ShowRB_V.get() == 1 and len(T2VRline) > 4:   # Write the ref trace B if active
        ca.create_line(T2VRline, fill=COLORtraceR2, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() == 1:
            Peak_label = ' {0:.2f} '.format(PeakdbRB) + ',' + ' {0:.1f} '.format(PeakfreqRB)
            ca.create_text(PeakxRB, PeakyRB, text=Peak_label, fill=COLORtraceR2, anchor="w", font=("arial", 8 ))
    if ShowRA_P.get() == 1 and len(T1PRline) > 4:   # Write the ref trace A if active
        ca.create_line(T1PRline, fill=COLORtraceR3, smooth=SmoothBool, splinestep=5)
    if ShowRB_P.get() == 1 and len(T2PRline) > 4:   # Write the ref trace A if active
        ca.create_line(T2PRline, fill=COLORtraceR4, smooth=SmoothBool, splinestep=5)
    if ShowMath.get() > 0 and len(TMline) > 4:   # Write the Math trace if active
        ca.create_line(TMline, fill=COLORtrace5, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() ==1:
            Peak_label = ' {0:.2f} '.format(PeakMdb) + ',' + ' {0:.1f} '.format(PeakfreqM)
            ca.create_text(PeakxM, PeakyM, text=Peak_label, fill=COLORtrace5, anchor="w", font=("arial", 8 ))
    if ShowRMath.get() == 1 and len(TRMline) > 4:   # Write the ref math trace if active
        ca.create_line(TRMline, fill=COLORtraceR5, smooth=SmoothBool, splinestep=5)
        if ShowMarker.get() ==1:
            Peak_label = ' {0:.2f} '.format(PeakRMdb) + ',' + ' {0:.1f} '.format(PeakfreqRM)
            ca.create_text(PeakxRM, PeakyRM, text=Peak_label, fill=COLORtraceR5, anchor="w", font=("arial", 8 ))
    # General information on top of the grid

    txt = "    Sample rate: " + str(SAMPLErate)
    txt = txt + "    FFT samples: " + str(SMPfft)

    if FFTwindow.get() == 0:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 1:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 2:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 3:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 4:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 5:
        txt = txt + "   " + FFTwindowname
    if FFTwindow.get() == 6:
        txt = txt + "   " + FFTwindowname
        
    x = X0L
    y = 12
    idTXT = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext)

    # Start and stop frequency and dB/div and trace mode
    StartValue = float(int(STARTfrequency.get())*int(MulXEntry.get()))
    StopValue = float(int(STOPfrequency.get())*int(MulXEntry.get()))
    if StartValue >= 1000000:
        StartValue = float(StartValue/1000000)
        Startlabel = str(StartValue)+"MHz"
    elif StartValue >= 1000:
        StartValue = StartValue/1000
        Startlabel = str(StartValue)+"KHz"
    else:
        Startlabel = str(StartValue)+"Hz"
    if StopValue >= 1000000:
        StopValue = float(StopValue/1000000)
        Stoplabel = str(StopValue)+"MHz"
    elif StopValue >= 1000:
        StopValue = StopValue/1000
        Stoplabel = str(StopValue)+"KHz"
    else:
        Stoplabel = str(StopValue)+"Hz"
    txt = Startlabel + " to " + Stoplabel
    txt = txt +  "  " + str(DBdivlist[DBdivindex.get()]) + " dB/div"
    txt = txt + "  Level: " + str(DBlevel.get()) + " dB "
    EffBandWidth = FFTbandwidth*int(MulXEntry.get())
    txt = txt + "  FFT Bandwidth =" + ' {0:.2f} '.format(EffBandWidth)
    
    x = X0L
    y = Y0T+GRH+23
    idTXT = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext)
    
    if TRACEmode.get() == 1:
        txt ="Normal mode "

    if TRACEmode.get() == 2:
        txt = "Peak hold mode "
    
    if TRACEmode.get() == 3:
        txt = "Power average  mode (" + str(TRACEaverage.get()) + ") " 

    if ZEROstuffing.get() > 0:
        txt = txt + "Zero Stuffing = " + str(ZEROstuffing.get())
    # Runstatus and level information
    if (RUNstatus == 0) and (SingleShot.get() == 0):
        txt = txt + "  Stopped "
    elif SingleShot.get() == 1:
        txt = txt + "  Single Shot Trace "
    else:
        txt = txt + "  Running "
    if FSweepMode.get() > 0:
        txt = txt + "  Freq Step = " + str(LoopNum)
    x = X0L
    y = Y0T+GRH+34
    IDtxt  = ca.create_text (x, y, text=txt, anchor=W, fill=COLORtext)

def INITIALIZEstart():
    global SMPfft
    global SMPfftpwrTwo
    global TRACEreset, TRACEmode, LoopNum, FSweepMode, FSweepCont

    # First some subroutines to set specific variables
    SMPfft = 2 ** int(SMPfftpwrTwo.get()) # Calculate the number of FFT samples from SMPfftpwrtwo

    CALCFFTwindowshape()
    if TRACEmode.get() == 1 and TRACEreset == False:
        TRACEreset = True               # Clear the memory for averaging or peak
    if TRACEmode.get() == 2 and LoopNum == 1 and FSweepMode.get() > 0 and FSweepCont.get() == 0:
        TRACEreset = True               # Clear the memory for peak hold when using sweep generator
def CALCFFTwindowshape():           # Make the FFTwindowshape for the windowing function
    global FFTbandwidth             # The FFT bandwidth
    global FFTwindow                # Which FFT window number is selected
    global FFTwindowname            # The name of the FFT window function
    global FFTwindowshape           # The window shape
    global SAMPLErate               # The sample rate
    global SMPfft                   # Number of FFT samples
    
    # FFTname and FFTbandwidth in milliHz
    FFTwindowname = "No such window"
    FFTbw = 0
    
    if FFTwindow.get() == 0:
        FFTwindowname = " Rectangular (no) window (B=1) "
        FFTbw = 1.0

    if FFTwindow.get() == 1:
        FFTwindowname = " Cosine window (B=1.24) "
        FFTbw = 1.24

    if FFTwindow.get() == 2:
        FFTwindowname = " Triangular window (B=1.33) "
        FFTbw = 1.33

    if FFTwindow.get() == 3:
        FFTwindowname = " Hann window (B=1.5) "
        FFTbw = 1.5

    if FFTwindow.get() == 4:
        FFTwindowname = " Blackman window (B=1.73) "
        FFTbw = 1.73

    if FFTwindow.get() == 5:
        FFTwindowname = " Nuttall window (B=2.02) "
        FFTbw = 2.02

    if FFTwindow.get() == 6:
        FFTwindowname = " Flat top window (B=3.77) "
        FFTbw = 3.77

    FFTbandwidth = int(FFTbw * SAMPLErate / float(SMPfft)) 

    # Calculate the shape
    FFTwindowshape = numpy.ones(SMPfft)         # Initialize with ones

    # m = 0                                       # For calculation of correction factor, furhter no function

    n = 0
    while n < SMPfft:

        # Cosine window function
        # medium-dynamic range B=1.24
        if FFTwindow.get() == 1:
            w = math.sin(math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 1.571

        # Triangular non-zero endpoints
        # medium-dynamic range B=1.33
        if FFTwindow.get() == 2:
            w = (2.0 / SMPfft) * ((SMPfft/ 2.0) - abs(n - (SMPfft - 1) / 2.0))
            FFTwindowshape[n] = w * 2.0

        # Hann window function
        # medium-dynamic range B=1.5
        if FFTwindow.get() == 3:
            w = 0.5 - 0.5 * math.cos(2 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 2.000

        # Blackman window, continuous first derivate function
        # medium-dynamic range B=1.73
        if FFTwindow.get() == 4:
            w = 0.42 - 0.5 * math.cos(2 * math.pi * n / (SMPfft - 1)) + 0.08 * math.cos(4 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 2.381

        # Nuttall window, continuous first derivate function
        # high-dynamic range B=2.02
        if FFTwindow.get() == 5:
            w = 0.355768 - 0.487396 * math.cos(2 * math.pi * n / (SMPfft - 1)) + 0.144232 * math.cos(4 * math.pi * n / (SMPfft - 1))- 0.012604 * math.cos(6 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 2.811

        # Flat top window, 
        # medium-dynamic range, extra wide bandwidth B=3.77
        if FFTwindow.get() == 6:
            w = 1.0 - 1.93 * math.cos(2 * math.pi * n / (SMPfft - 1)) + 1.29 * math.cos(4 * math.pi * n / (SMPfft - 1))- 0.388 * math.cos(6 * math.pi * n / (SMPfft - 1)) + 0.032 * math.cos(8 * math.pi * n / (SMPfft - 1))
            FFTwindowshape[n] = w * 1.000
        
        # m = m + w / SMPfft                          # For calculation of correction factor
        n = n + 1

    # if m > 0:                                     # For calculation of correction factor
    #     print "correction 1/m: ", 1/m             # For calculation of correction factor

def ConnectDevice():
    global devx, DevID, Both, CHA, CHB, bcon
    try:
        devx = Smu()
        brdnum = len(devx.devices)
        channum = len(devx.channels)
        CHA = devx.channels['A']
        CHA.set_mode('d')           # Put CHA in Hi Z mode
        CHB = devx.channels['B']
        CHB.set_mode('d')           # Put CHB in Hi Z mode
        DevID = devx.serials[0]
        print DevID
        ADsignal1 = []              # Ain signal array 
        Both = devx.devices[0]
        bcon.configure(text="Conn", bg="green")
        if channum == 4:
            CHC = devx.channels['C']
            CHC.set_mode('d')
            CHD = devx.channels['D']
            CHD.set_mode('d')
            DevTwo = devx.serials[1]
            print DevTwo
            BTwo = devx.devices[1]
    except:
        print 'No Device plugged IN!'
        DevID = "No Device"
        bcon.configure(text="Recon", bg="red")
    
# ================ Make Screen ==========================

frame2r = Frame(root, borderwidth=5, relief=RIDGE)
frame2r.pack(side=RIGHT, expand=1, fill=Y)

frame1 = Frame(root, borderwidth=5, relief=RIDGE)
frame1.pack(side=TOP, expand=1, fill=X)

frame2 = Frame(root, background="black", borderwidth=5, relief=RIDGE)
frame2.pack(side=TOP, expand=YES, fill=BOTH)

frame3 = Frame(root, borderwidth=5, relief=RIDGE)
frame3.pack(side=TOP, expand=1, fill=X)

ca = Canvas(frame2, width=CANVASwidth, height=CANVASheight, background=COLORcanvas)
ca.pack(side=TOP)

# top menu buttons
SingleShot = IntVar(0) # variable for Single Shot sweeps
Modemenu = Menubutton(frame1, text="Mode", relief=RAISED)
Modemenu.menu = Menu(Modemenu, tearoff = 0 )
Modemenu["menu"]  = Modemenu.menu
Modemenu.menu.add_command(label="Normal mode", command=BNormalmode)
Modemenu.menu.add_command(label="Peak hold", command=BPeakholdmode)
Modemenu.menu.add_command(label="Average", command=BAveragemode)
Modemenu.menu.add_checkbutton(label='SingleShot', variable=SingleShot)
Modemenu.pack(side=LEFT)
#
FFTwindow = IntVar(0)   # FFT window function variable
FFTwindmenu = Menubutton(frame1, text="FFTwindow", relief=RAISED)
FFTwindmenu.menu = Menu(FFTwindmenu, tearoff = 0 )
FFTwindmenu["menu"]  = FFTwindmenu.menu
FFTwindmenu.menu.add_radiobutton(label='Rectangular window (B=1)', variable=FFTwindow, value=0)
FFTwindmenu.menu.add_radiobutton(label='Cosine window (B=1.24)', variable=FFTwindow, value=1)
FFTwindmenu.menu.add_radiobutton(label='Triangular window (B=1.33)', variable=FFTwindow, value=2)
FFTwindmenu.menu.add_radiobutton(label='Hann window (B=1.5)', variable=FFTwindow, value=3)
FFTwindmenu.menu.add_radiobutton(label='Blackman window (B=1.73)', variable=FFTwindow, value=4)
FFTwindmenu.menu.add_radiobutton(label='Nuttall window (B=2.02)', variable=FFTwindow, value=5)
FFTwindmenu.menu.add_radiobutton(label='Flat top window (B=3.77)', variable=FFTwindow, value=6)
FFTwindmenu.pack(side=LEFT)
#
b = Button(frame1, text="-Samples", width=Buttonwidth2, command=Bsamples1)
b.pack(side=LEFT)
b = Button(frame1, text="+Samples", width=Buttonwidth2, command=Bsamples2)
b.pack(side=LEFT)
#
b = Button(frame1, text="Exit", width=4, command=Bcloseexit)
b.pack(side=RIGHT)
b = Button(frame1, text="Stop", width=4, bg="red", command=BStop)
b.pack(side=RIGHT)
b = Button(frame1, text="Run", width=5, bg="green", command=BStart)
b.pack(side=RIGHT)
PwrBt = Button(frame1, text="PWR-ON", width=7, bg="green", command=BPower)
PwrBt.pack(side=RIGHT)
# Show channels menu
ShowC1_V = IntVar(0)   # curves to display variables
ShowC1_P = IntVar(0)
ShowC2_V = IntVar(0)
ShowC2_P = IntVar(0)
ShowMarker = IntVar(0)
ShowRA_V = IntVar(0)
ShowRA_P = IntVar(0)
ShowRB_V = IntVar(0)
ShowRB_P = IntVar(0)
ShowMath = IntVar(0)
ShowRMath = IntVar(0)
#
Showmenu = Menubutton(frame1, text="Curves", relief=RAISED)
Showmenu.menu = Menu(Showmenu, tearoff = 0 )
Showmenu["menu"] = Showmenu.menu
Showmenu.menu.add_command(label="-Show-", command=donothing)
Showmenu.menu.add_command(label="All", command=BShowCurvesAll)
Showmenu.menu.add_command(label="None", command=BShowCurvesNone)
Showmenu.menu.add_checkbutton(label='CA-dBV', variable=ShowC1_V)
Showmenu.menu.add_checkbutton(label='CB-dBV', variable=ShowC2_V)
Showmenu.menu.add_checkbutton(label='Phase A-B', variable=ShowC1_P)
Showmenu.menu.add_checkbutton(label='Phase B-A', variable=ShowC2_P)
Showmenu.menu.add_checkbutton(label='Marker', variable=ShowMarker)
Showmenu.menu.add_separator()
Showmenu.menu.add_radiobutton(label='None', variable=ShowMath, value=0)
Showmenu.menu.add_radiobutton(label='CA-dB - CB-dB', variable=ShowMath, value=1)
Showmenu.menu.add_radiobutton(label='CB-dB - CA-dB', variable=ShowMath, value=2)
Showmenu.menu.add_separator()
Showmenu.menu.add_checkbutton(label='RA-dBV', variable=ShowRA_V)
Showmenu.menu.add_checkbutton(label='RB-dBV', variable=ShowRB_V)
Showmenu.menu.add_checkbutton(label='RPhase A-B', variable=ShowRA_P)
Showmenu.menu.add_checkbutton(label='RPhase B-A', variable=ShowRB_P)
Showmenu.menu.add_checkbutton(label='Math', variable=ShowRMath)
Showmenu.pack(side=RIGHT)
# Sampling controls Widgets
fminlab2 = Label(frame1, text="KHz")
fminlab2.pack(side=RIGHT)
FminEntry = Entry(frame1, width=4)
FminEntry.bind("<Return>", BsetAD9833)
FminEntry.pack(side=RIGHT)
FminEntry.delete(0,"end")
FminEntry.insert(0,25)
fminlab = Label(frame1, text="Fmin")
fminlab.pack(side=RIGHT)
#
MulXEntry = Entry(frame1, width=4)
MulXEntry.bind("<Return>", BsetAD9833)
MulXEntry.pack(side=RIGHT)
MulXEntry.delete(0,"end")
MulXEntry.insert(0,1000)
mulxlab = Label( frame1, text = "Mul X")
mulxlab.pack(side=RIGHT)
# right side drop down menu buttons
dropmenu = Frame( frame2r )
dropmenu.pack(side=TOP)
bcon = Button(dropmenu, text="Recon", width=5, command=ConnectDevice)
bcon.pack(side=LEFT, anchor=W)
# File menu
Filemenu = Menubutton(dropmenu, text="File", relief=RAISED)
Filemenu.menu = Menu(Filemenu, tearoff = 0 )
Filemenu["menu"] = Filemenu.menu
Filemenu.menu.add_command(label="Save Config", command=BSaveConfig)
Filemenu.menu.add_command(label="Load Config", command=BLoadConfig)
Filemenu.menu.add_command(label="Save Screen", command=BSaveScreen)
Filemenu.menu.add_command(label="Save Data", command=BCSVfile)
Filemenu.menu.add_command(label="Help", command=BHelp)
Filemenu.pack(side=LEFT, anchor=W)
#
SmoothCurves = IntVar(0)
CutDC = IntVar(0)
Optionmenu = Menubutton(dropmenu, text="Options", relief=RAISED)
Optionmenu.menu = Menu(Optionmenu, tearoff = 0 )
Optionmenu["menu"]  = Optionmenu.menu
Optionmenu.menu.add_checkbutton(label='Smooth', variable=SmoothCurves)
Optionmenu.menu.add_checkbutton(label='Cut-DC', variable=CutDC)
Optionmenu.menu.add_command(label="Store trace", command=BSTOREtrace)
Optionmenu.menu.add_command(label="Setup", command=BSetup)
Optionmenu.menu.add_command(label="Screen setup", command=BScreensetup)
Optionmenu.pack(side=LEFT, anchor=W)
#
# now AWG A
awg1lab = Label(frame2r, text="AWG CH A")
awg1lab.pack(side=TOP)
# AWG enable sub frame
AWGAMode = IntVar(0)   # AWG A mode variable
AWGATerm = IntVar(0)   # AWG A termination variable
AWGAShape = IntVar(0)  # AWG A Wave shape variable
awg1eb = Frame( frame2r )
awg1eb.pack(side=TOP)
ModeAMenu = Menubutton(awg1eb, text="Mode", relief=RAISED)
ModeAMenu.menu = Menu(ModeAMenu, tearoff = 0 )
ModeAMenu["menu"] = ModeAMenu.menu
ModeAMenu.menu.add_command(label="-Mode-", command=donothing)
ModeAMenu.menu.add_radiobutton(label="SVMI", variable=AWGAMode, value=0)
ModeAMenu.menu.add_radiobutton(label="SIMV", variable=AWGAMode, value=1)
ModeAMenu.menu.add_radiobutton(label="Hi-Z", variable=AWGAMode, value=2)
ModeAMenu.menu.add_radiobutton(label="Split I/O", variable=AWGAMode, value=3)
ModeAMenu.menu.add_separator()
ModeAMenu.menu.add_command(label="-Term-", command=donothing)
ModeAMenu.menu.add_radiobutton(label="Open", variable=AWGATerm, value=0)
ModeAMenu.menu.add_radiobutton(label="To GND", variable=AWGATerm, value=1)
ModeAMenu.menu.add_radiobutton(label="To 2.5V", variable=AWGATerm, value=2)
ModeAMenu.pack(side=LEFT, anchor=W)
ShapeAMenu = Menubutton(awg1eb, text="Shape", relief=RAISED)
ShapeAMenu.menu = Menu(ShapeAMenu, tearoff = 0 )
ShapeAMenu["menu"] = ShapeAMenu.menu
ShapeAMenu.menu.add_radiobutton(label="DC", variable=AWGAShape, value=0)
ShapeAMenu.menu.add_radiobutton(label="Sine", variable=AWGAShape, value=1)
ShapeAMenu.menu.add_radiobutton(label="Triangle", variable=AWGAShape, value=2)
ShapeAMenu.menu.add_radiobutton(label="Sawtooth", variable=AWGAShape, value=3)
ShapeAMenu.menu.add_radiobutton(label="Square", variable=AWGAShape, value=4)
ShapeAMenu.menu.add_radiobutton(label="StairStep", variable=AWGAShape, value=5)
ShapeAMenu.menu.add_radiobutton(label="Custom", variable=AWGAShape, value=6)
ShapeAMenu.pack(side=LEFT, anchor=W)
#
awg1ampl = Frame( frame2r )
awg1ampl.pack(side=TOP)
AWGAAmplEntry = Entry(awg1ampl, width=5)
AWGAAmplEntry.bind("<Return>", BAWGAAmpl)
AWGAAmplEntry.pack(side=LEFT, anchor=W)
AWGAAmplEntry.delete(0,"end")
AWGAAmplEntry.insert(0,0.0)
amp1lab = Label(awg1ampl, text="Min")
amp1lab.pack(side=LEFT, anchor=W)
#
awg1off = Frame( frame2r )
awg1off.pack(side=TOP)
AWGAOffsetEntry = Entry(awg1off, width=5)
AWGAOffsetEntry.bind("<Return>", BAWGAOffset)
AWGAOffsetEntry.pack(side=LEFT, anchor=W)
AWGAOffsetEntry.delete(0,"end")
AWGAOffsetEntry.insert(0,0.0)
off1lab = Label(awg1off, text="Max")
off1lab.pack(side=LEFT, anchor=W)
# AWG Frequency sub frame
awg1freq = Frame( frame2r )
awg1freq.pack(side=TOP)
AWGAFreqEntry = Entry(awg1freq, width=7)
AWGAFreqEntry.bind("<Return>", BAWGAFreq)
AWGAFreqEntry.pack(side=LEFT, anchor=W)
AWGAFreqEntry.delete(0,"end")
AWGAFreqEntry.insert(0,0.0)
freq1lab = Label(awg1freq, text="Freq")
freq1lab.pack(side=LEFT, anchor=W)
# AWG Phase sub frame
awg1phase = Frame( frame2r )
awg1phase.pack(side=TOP)
AWGAPhaseEntry = Entry(awg1phase, width=5)
AWGAPhaseEntry.bind("<Return>", BAWGAPhase)
AWGAPhaseEntry.pack(side=LEFT, anchor=W)
AWGAPhaseEntry.delete(0,"end")
AWGAPhaseEntry.insert(0,0)
phase1lab = Label(awg1phase, text="Phase")
phase1lab.pack(side=LEFT, anchor=W)
# AWG duty cycle frame
awg1dc = Frame( frame2r )
awg1dc.pack(side=TOP)
AWGADutyCycleEntry = Entry(awg1dc, width=5)
AWGADutyCycleEntry.bind("<Return>", BAWGADutyCycle)
AWGADutyCycleEntry.pack(side=LEFT, anchor=W)
AWGADutyCycleEntry.delete(0,"end")
AWGADutyCycleEntry.insert(0,50)
duty1lab = Label(awg1dc, text="%")
duty1lab.pack(side=LEFT, anchor=W)

# now AWG B
awg2lab = Label(frame2r, text="AWG CH B")
awg2lab.pack(side=TOP)
# AWG enable sub frame
AWGBMode = IntVar(0)   # AWG B mode variable
AWGBTerm = IntVar(0)   # AWG B termination variable
AWGBShape = IntVar(0)  # AWG B Wave shape variable
awg2eb = Frame( frame2r )
awg2eb.pack(side=TOP)
ModeBMenu = Menubutton(awg2eb, text="Mode", relief=RAISED)
ModeBMenu.menu = Menu(ModeBMenu, tearoff = 0 )
ModeBMenu["menu"] = ModeBMenu.menu
ModeBMenu.menu.add_command(label="-Mode-", command=donothing)
ModeBMenu.menu.add_radiobutton(label="SVMI", variable=AWGBMode, value=0)
ModeBMenu.menu.add_radiobutton(label="SIMV", variable=AWGBMode, value=1)
ModeBMenu.menu.add_radiobutton(label="Hi-Z", variable=AWGBMode, value=2)
ModeBMenu.menu.add_radiobutton(label="Split I/O", variable=AWGBMode, value=3)
ModeBMenu.menu.add_separator()
ModeBMenu.menu.add_command(label="-Term-", command=donothing)
ModeBMenu.menu.add_radiobutton(label="Open", variable=AWGBTerm, value=0)
ModeBMenu.menu.add_radiobutton(label="To GND", variable=AWGBTerm, value=1)
ModeBMenu.menu.add_radiobutton(label="To 2.5V", variable=AWGBTerm, value=2)
ModeBMenu.pack(side=LEFT, anchor=W)
ShapeBMenu = Menubutton(awg2eb, text="Shape", relief=RAISED)
ShapeBMenu.menu = Menu(ShapeBMenu, tearoff = 0 )
ShapeBMenu["menu"] = ShapeBMenu.menu
ShapeBMenu.menu.add_radiobutton(label="DC", variable=AWGBShape, value=0)
ShapeBMenu.menu.add_radiobutton(label="Sine", variable=AWGBShape, value=1)
ShapeBMenu.menu.add_radiobutton(label="Triangle", variable=AWGBShape, value=2)
ShapeBMenu.menu.add_radiobutton(label="Sawtooth", variable=AWGBShape, value=3)
ShapeBMenu.menu.add_radiobutton(label="Square", variable=AWGBShape, value=4)
ShapeBMenu.menu.add_radiobutton(label="StairStep", variable=AWGBShape, value=5)
ShapeBMenu.menu.add_radiobutton(label="Custom", variable=AWGBShape, value=6)
ShapeBMenu.pack(side=LEFT, anchor=W)
#
awg2ampl = Frame( frame2r )
awg2ampl.pack(side=TOP)
AWGBAmplEntry = Entry(awg2ampl, width=5)
AWGBAmplEntry.bind("<Return>", BAWGBAmpl)
AWGBAmplEntry.pack(side=LEFT, anchor=W)
AWGBAmplEntry.delete(0,"end")
AWGBAmplEntry.insert(0,0.0)
amp2lab = Label(awg2ampl, text="Min")
amp2lab.pack(side=LEFT, anchor=W)
#
awg2off = Frame( frame2r )
awg2off.pack(side=TOP)
AWGBOffsetEntry = Entry(awg2off, width=5)
AWGBOffsetEntry.bind("<Return>", BAWGBOffset)
AWGBOffsetEntry.pack(side=LEFT, anchor=W)
AWGBOffsetEntry.delete(0,"end")
AWGBOffsetEntry.insert(0,0.0)
off2lab = Label(awg2off, text="Max")
off2lab.pack(side=LEFT, anchor=W)
# AWG Frequency sub frame
awg2freq = Frame( frame2r )
awg2freq.pack(side=TOP)
AWGBFreqEntry = Entry(awg2freq, width=7)
AWGBFreqEntry.bind("<Return>", BAWGBFreq)
AWGBFreqEntry.pack(side=LEFT, anchor=W)
AWGBFreqEntry.delete(0,"end")
AWGBFreqEntry.insert(0,0.0)
freq2lab = Label(awg2freq, text="Freq")
freq2lab.pack(side=LEFT, anchor=W)
# AWG Phase sub frame
awg2phase = Frame( frame2r )
awg2phase.pack(side=TOP)
AWGBPhaseEntry = Entry(awg2phase, width=5)
AWGBPhaseEntry.bind("<Return>", BAWGBPhase)
AWGBPhaseEntry.pack(side=LEFT, anchor=W)
AWGBPhaseEntry.delete(0,"end")
AWGBPhaseEntry.insert(0,0)
phase2lab = Label(awg2phase, text="Phase")
phase2lab.pack(side=LEFT, anchor=W)
# AWG duty cycle frame
awg2dc = Frame( frame2r )
awg2dc.pack(side=TOP)
AWGBDutyCycleEntry = Entry(awg2dc, width=5)
AWGBDutyCycleEntry.bind("<Return>", BAWGBDutyCycle)
AWGBDutyCycleEntry.pack(side=LEFT, anchor=W)
AWGBDutyCycleEntry.delete(0,"end")
AWGBDutyCycleEntry.insert(0,50)
duty2lab = Label(awg2dc, text="%")
duty2lab.pack(side=LEFT, anchor=W)

AWGSync = IntVar(0) # Sync start both AWG channels
awgsync = Checkbutton(frame2r, text="Sync AWG", variable=AWGSync, command=BAWGSync)
awgsync.pack(side=TOP)
# sweep generator mode menu buttons
FSweepMode = IntVar(0)   # Sweep Generator mode
FSweepCont = IntVar(0)
FSweepmenu = Menubutton(frame2r, text="Sweep Gen", relief=RAISED)
FSweepmenu.menu = Menu(FSweepmenu, tearoff = 0 )
FSweepmenu["menu"]  = FSweepmenu.menu
FSweepmenu.menu.add_radiobutton(label='None', variable=FSweepMode, value=0)
FSweepmenu.menu.add_radiobutton(label='CHA', variable=FSweepMode, value=1)
FSweepmenu.menu.add_radiobutton(label='CHB', variable=FSweepMode, value=2)
FSweepmenu.menu.add_separator()
FSweepmenu.menu.add_command(label='Sweep Steps', command=BSweepsteps)
FSweepmenu.menu.add_separator()
FSweepmenu.menu.add_radiobutton(label='Single', variable=FSweepCont, value=0)
FSweepmenu.menu.add_radiobutton(label='Continuous', variable=FSweepCont, value=1)
FSweepmenu.pack(side=TOP)
# MiniGen wigets
Omode = IntVar(0)
minigenfr = Frame( frame2r )
minigenfr.pack(side=TOP)
MiniGenMenu = Menubutton(minigenfr, text="MiniGen", relief=RAISED)
MiniGenMenu.menu = Menu(MiniGenMenu, tearoff = 0 )
MiniGenMenu["menu"] = MiniGenMenu.menu
MiniGenMenu.menu.add_radiobutton(label="Sine", variable=Omode, value=0, command=SetAD9833)
MiniGenMenu.menu.add_radiobutton(label="Triangle", variable=Omode, value=2, command=SetAD9833)
MiniGenMenu.menu.add_radiobutton(label="Square", variable=Omode, value=40, command=SetAD9833)
MiniGenMenu.menu.add_radiobutton(label="Square/2", variable=Omode, value=32, command=SetAD9833)
MiniGenMenu.pack(side=LEFT, anchor=W)
MiniGenEntry = Entry(minigenfr, width=5)
MiniGenEntry.bind("<Return>", BsetAD9833)
MiniGenEntry.pack(side=LEFT, anchor=W)
MiniGenEntry.delete(0,"end")
MiniGenEntry.insert(0,10.0)
# input probe wigets
prlab = Label(frame2r, text="Input Gain Offset")
prlab.pack(side=TOP)
# Input Probes sub frame 
ProbeA = Frame( frame2r )
ProbeA.pack(side=TOP)
gain1lab = Label(ProbeA, text="CA")
gain1lab.pack(side=LEFT)
CHAVGainEntry = Entry(ProbeA, width=5) # command=Bprobe)
CHAVGainEntry.pack(side=LEFT)
CHAVGainEntry.delete(0,"end")
CHAVGainEntry.insert(0,1.0)
CHAVOffsetEntry = Entry(ProbeA, width=5) # command=Bprobe)
CHAVOffsetEntry.pack(side=LEFT)
CHAVOffsetEntry.delete(0,"end")
CHAVOffsetEntry.insert(0,0.0)
#
ProbeB = Frame( frame2r )
ProbeB.pack(side=TOP)
gain2lab = Label(ProbeB, text="CB")
gain2lab.pack(side=LEFT)
CHBVGainEntry = Entry(ProbeB, width=5) # command=Bprobe)
CHBVGainEntry.pack(side=LEFT)
CHBVGainEntry.delete(0,"end")
CHBVGainEntry.insert(0,1.0)
CHBVOffsetEntry = Entry(ProbeB, width=5) # command=Bprobe)
CHBVOffsetEntry.pack(side=LEFT)
CHBVOffsetEntry.delete(0,"end")
CHBVOffsetEntry.insert(0,0.0)
# add ADI logo Don't mess with this bit map data!
ADIlogo = """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"""
logo = PhotoImage(data=ADIlogo)
ADI1 = Label(frame2r, image=logo, anchor= "sw", height=49, width=116, compound="top")
ADI1.pack(side=TOP)

#
b = Button(frame3, text="Startfreq", width=Buttonwidth2, command=BStartfrequency)
b.pack(side=LEFT)

b = Button(frame3, text="Stopfreq", width=Buttonwidth2, command=BStopfrequency)
b.pack(side=LEFT)

b = Button(frame3, text="+dB/div", width=Buttonwidth2, command=BDBdiv2)
b.pack(side=RIGHT)

b = Button(frame3, text="-dB/div", width=Buttonwidth2, command=BDBdiv1)
b.pack(side=RIGHT)

b = Button(frame3, text="LVL+10", width=Buttonwidth2, command=Blevel4)
b.pack(side=RIGHT)

b = Button(frame3, text="LVL-10", width=Buttonwidth2, command=Blevel3)
b.pack(side=RIGHT)

b = Button(frame3, text="LVL+1", width=Buttonwidth2, command=Blevel2)
b.pack(side=RIGHT)

b = Button(frame3, text="LVL-1", width=Buttonwidth2, command=Blevel1)
b.pack(side=RIGHT)

#===== Initalize device ======
AWGAMode.set(2)
AWGBMode.set(2)
ConnectDevice()
# ================ Call main routine ===============================
root.update()               # Activate updated screens
#
# Start sampling
Analog_In()
 


